<?php

namespace App\Console\Commands;

use Exception;
use Carbon\Carbon;
use App\Mail\TableMail;
use Illuminate\Http\Request;
use App\Services\BaseService;
use App\Services\ConfigService;
use Illuminate\Console\Command;
use App\Services\ScheduleService;
use App\Repositories\BaseRepository;
use Illuminate\Support\Facades\Mail;

class DailyCheck extends Command
{
    //使用者郵件
    private const UsersMail = [
        'Annika' => 'annika@sunstar-dt.com',
        'Dante' => 'dante@sunstar-dt.com',
        'Joe' => 'joe@sunstar-dt.com',
        'Sam' => 'sam@sunstar-dt.com'
    ];

    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'daily:check';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = '每日檢查';

    protected $scheduleService;
    protected $baseService;

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $this->scheduleService = new ScheduleService();
        $baseRepository = new BaseRepository();
        $this->baseService = new BaseService($baseRepository, new ConfigService($baseRepository));

        $this->SafetyStockCheck();
        $this->StockUnusualCheck();
        $this->CustomerComplaintCheck();
        $this->ProductionDiaryCheck();
    }

    /**
     * 安全庫存檢查
     *
     * @return void
     */
    private function SafetyStockCheck()
    {
        try {
            //建立指定料號的請求
            $request = new Request();
            $request->replace(['designation' => 1]);

            $db = $this->baseService->GetSafetyStockHistory($request);
            $data = array();
            if (!empty($db)) {
                $index = 0;
                foreach ($db as $item) {
                    //庫存剩餘天數<40 且 無採購單號 且 無覆核
                    if ($item->remaining_day < 40 && empty($item->purchase_no) && empty($item->signinger)) {
                        $data[] = [
                            ++$index,
                            $item->supplier,
                            $item->item_no,
                            $item->item_name,
                            number_format($item->stock),
                            number_format($item->daily_sales),
                            number_format($item->remaining_day),
                            number_format($item->suggest),
                            $item->inventory_date,
                            number_format($item->inventory_num),
                        ];
                    }
                }
            }

            if (!empty($data)) {
                $mailData =
                    [
                        'subject' => '庫存建議採購',
                        'caption' => '商品安庫表',
                        'thead' => ['序', '供應商', '商品料號', '商品名稱', '庫存', '每日銷售(30天平均)', '庫存剩餘天數', '建議採購(30天量)', '最後盤點日期', '最後盤點數量'],
                        'tbody' => $data
                    ];
                Mail::to(self::UsersMail['Dante'])
                    ->cc(self::UsersMail['Dante'])
                    ->send(new TableMail($mailData));
            }
            $this->scheduleService->CreateOnSuccessLog($this->signature, '安全庫存檢查');
        } catch (Exception $ex) {
            $this->scheduleService->CreateOnFailureLog($this->signature, '安全庫存檢查', $ex->getMessage());
        }
    }

    /**
     * 庫存異常檢查
     *
     * @return void
     */
    private function StockUnusualCheck()
    {
        try {
            $db = $this->baseService->GetStockUnusual();
            $data = array();
            if (!empty($db)) {
                $index = 0;
                foreach ($db as $value) {
                    $data[] = [
                        ++$index,
                        $value->ICODE,
                        $value->INAME,
                        $value->STNAM,
                        number_format($value->SNUMB)
                    ];
                }
            }

            if (!empty($data)) {
                $mailData =
                    [
                        'subject' => '庫存異常',
                        'caption' => '庫存異常清單',
                        'thead' => ['序', '料號', '品名', '倉別', '庫存數'],
                        'tbody' => $data
                    ];
                Mail::to(self::UsersMail['Dante'])
                    ->cc(self::UsersMail['Dante'])
                    ->send(new TableMail($mailData));
            }
            $this->scheduleService->CreateOnSuccessLog($this->signature, '庫存異常檢查');
        } catch (Exception $ex) {
            $this->scheduleService->CreateOnFailureLog($this->signature, '庫存異常檢查', $ex->getMessage());
        }
    }

    /**
     * 客訴單彙整
     *
     * @return void
     */
    private function CustomerComplaintCheck()
    {
        try {
            $request = new Request();
            $request->replace(['start_date' => Carbon::today()->addDays(-15)->format('Y-m-d')]);
            $request->replace(['end_date' => Carbon::today()->format('Y-m-d')]);
            $data = $this->baseService->GetCustomerComplaint($request);

            if (!empty($data['data'])) {
                $tmp = $data['data'];
                $pending = count($tmp->where('status', '待處理'));
                $processing = count($tmp->where('status', '處理中'));

                if ($pending > 0 || $processing > 0) {
                    $mailData =
                        [
                            'subject' => '客訴單彙整',
                            'caption' => '客訴單清單',
                            'thead' => ['序', '狀態', '單數'],
                            'tbody' => [
                                ['1', '待處理', $pending],
                                ['2', '處理中', $processing]
                            ]
                        ];
                    Mail::to(self::UsersMail['Dante'])
                        ->cc(self::UsersMail['Dante'])
                        ->send(new TableMail($mailData));
                }
            }

            $this->scheduleService->CreateOnSuccessLog($this->signature, '客訴單彙整');
        } catch (Exception $ex) {
            $this->scheduleService->CreateOnFailureLog($this->signature, '客訴單彙整', $ex->getMessage());
        }
    }

    /**
     * 日報表檢查
     *
     * @return void
     */
    private function ProductionDiaryCheck()
    {
        try {
            //建立取得前一天日報表的請求
            $request = new Request();
            $request->replace([
                'start_date' => Carbon::today()->addDays(-1)->format('Y-m-d'),
                'end_date' => Carbon::today()->addDays(-1)->format('Y-m-d')
            ]);
            $db = $this->baseService->GetProductionDiary($request);
            if (!empty($db['data']) && count($db['data']) > 0) {
                $data = $db['data']->first();
                if (empty($data->status) || $data->status == 4) {
                    switch ($data->status) {
                        case 1:
                            $status = '無驗貨紀錄';
                            break;
                        case 2:
                            $status = '符合預期';
                            break;
                        case 3:
                            $status = '已覆核';
                            break;
                        case 4:
                            $status = '不符合預期';
                            break;
                        default:
                            $status = '未定義';
                            break;
                    }

                    $mailData =
                        [
                            'subject' => '昨日報表不符合預期',
                            'caption' => '日報表清單',
                            'thead' => ['日期', '預定完成件數', '預定完成時間', '實際完成件數', '實際完成時間', '狀態', '工作簡述'],
                            'tbody' => [
                                [
                                    $data->settle_date,
                                    $data->amount,
                                    $data->estimated_time,
                                    $data->checkCount,
                                    $data->lastCheckTime,
                                    $status,
                                    $data->remark
                                ]
                            ]
                        ];

                    Mail::to(self::UsersMail['Dante'])
                        ->cc(self::UsersMail['Dante'])
                        ->send(new TableMail($mailData));
                }
            }

            $this->scheduleService->CreateOnSuccessLog($this->signature, '日報表彙整');
        } catch (Exception $ex) {
            $this->scheduleService->CreateOnFailureLog($this->signature, '日報表彙整', $ex->getMessage());
        }
    }
}
