<?php

namespace App\Repositories;

use DateTime;
use Exception;
use App\Models\AdItem;
use App\Models\SafetyStock;
use App\Models\PackageShiftM;
use App\Models\OrderSampleFile;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\Query\Builder;

class BaseRepository
{
    //倉別清單
    private const stockNames = ['出貨倉', '主倉儲', '零售倉', '待調撥倉', '退貨倉'];
    //排除的特殊客戶。面交、邱大哥、亞銘燈飾、外包設計師、包材、半成品領料、其他
    private const excludeedCustomers = ['C001', 'C008', 'D001', 'F001', 'x001', 'x002', 'Z001'];
    //排除產品的大類。其他、包材
    private const excludeedSalesType = ['98', '99'];
    //贈品的料號
    public const bulbCode = ['10010009001', '10010009002', '10003009012'];
    //地板的料號
    public const floorCode = [
        '11010014005', '11010014006', '11010014007', '11010014008', '11010014015', '11010014020', '11010014021', '11010014022', '11010014024',
    ];

    /**
     * 取得指定區間的訂單資料
     *
     * @param string $sDate 開始日期
     * @param string $eDate 結束日期
     * @return \Illuminate\Support\Collection
     */
    public function GetOrder(string $sDate, string $eDate)
    {
        $result = DB::table('ORDET AS M')
            ->select('M.OCOD4', 'D.OICOD', 'D.OINAM', 'D.OCOD1', 'D.OINUM')
            ->where('M.ODAT2', '')
            ->where('M.OCANC', '!=', '訂單取消')
            ->whereBetween('M.ODATE', [$sDate, $eDate])
            ->join('ORSUB AS D', function ($join) {
                $join
                    ->on('D.OCOD1', 'M.OCOD1')
                    ->whereNotIn('D.OICOD', ['XX123', 'XX126']) //排除網路運費、折扣
                    ->whereNotBetween('D.OICOD', ['99010022001', '99010022020']); //排除包材
            })
            ->whereNotIn('M.OCCOD', self::excludeedCustomers)
            ->orderBy('D.OICOD')
            ->orderBy('D.OINAM');

        return $result->get();
    }

    /**
     * 取得面交訂單
     *
     * @param string $sDate 開始日期
     * @param string $eDate 結束日期
     * @param boolean $isHeavy 是否為重物
     * @return \Illuminate\Support\Collection
     */
    public function GetOrderFaceToFace(string $sDate, string $eDate, bool $isHeavy = false)
    {
        $result = DB::table('ORDET AS T')
            ->select('T.OCOD1', 'T.OBAK1', 'P.PCOD1')
            ->leftJoin('POSEIN AS P', 'P.PCOD2', 'T.OCOD1')
            ->leftJoin('PoseinCheckDataLog AS L', 'L.PCOD1', 'P.PCOD1')
            ->whereBetween('T.ODATE', [$sDate, $eDate])
            ->where('T.OCCOD', 'C001')
            ->where(function ($query) { //未有驗貨紀錄
                $query
                    ->whereNull('L.PCOD1')
                    ->orWhere('L.PCOD1', '');
            })
            ->distinct('T.OCOD1');
        if ($isHeavy) {
            $result->join('ORSUB AS D', function ($join) {
                $join
                    ->on('D.OCOD1', 'T.OCOD1')
                    ->whereBetween('D.OICOD', ['11010014001', '11010014024']);
            });
        }

        return $result->get();
    }


    /**
     * 取的指定日期的所有訂單
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @param boolean $isHeavy 是否為重物
     * @return \Illuminate\Support\Collection
     */
    public function GetOrderAll(string $sDate, string $eDate, bool $isHeavy = false)
    {
        $result = DB::table('ORDET AS M')
            ->select('M.OCOD1', 'M.OBAK1', 'M.OCANC', 'M.ODATE')
            ->whereBetween('M.ODATE', [$sDate, $eDate])
            ->whereNotIn('M.OCCOD', self::excludeedCustomers)
            ->distinct('M.OCOD1');
        if ($isHeavy) {
            $result->join('ORSUB AS D', function ($join) {
                $join
                    ->on('D.OCOD1', 'M.OCOD1')
                    ->whereBetween('D.OICOD', ['11010014001', '11010014024']);
            });
        }

        return $result->get();
    }

    /**
     * 取得指定訂單單號的所有產品
     *
     * @param array $orderList 訂單清單
     * @return \Illuminate\Support\Collection
     */
    public function GetOrderProduct(array $orderList)
    {
        if (empty($orderList)) {
            return collect();
        }
        $result = DB::table('ORSUB')
            ->select('OCOD1', 'OICOD', 'OINUM')
            ->whereIn('OCOD1', $orderList)
            ->orderBy('OCOD1')
            ->orderBy('OICOD');

        return $result->get();
    }

    /**
     * 取得未轉單資料
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetOrderUnprinted()
    {
        $result = DB::table('ORDET AS O')
            ->select('O.OCOD1', 'O.OCOD4', 'O.ODATE')
            ->leftJoin('POSEIN AS P', 'P.PCOD2', 'O.OCOD1')
            ->where('O.OCANC', '!=', '訂單取消')
            ->whereNull('P.PCOD2')
            ->orderBy('O.ODATE')
            ->orderBy('O.OCOD1');

        return $result->get();
    }

    /**
     * 取得各庫存資料
     * 
     * @param array $sitemList 料號清單
     * @return \Illuminate\Support\Collection
     */
    public function GetStock(array $sitemList = null)
    {
        $result = DB::table('STOC')
            ->selectRaw('SITEM, STNAM, SUM(SNUMB) AS SNUMB')
            ->whereIn('STNAM', self::stockNames)
            ->groupBy('STCOD', 'SITEM', 'STNAM')
            ->orderBy('SITEM')
            ->orderBy('STNAM');

        if (!empty($sitemList)) {
            $result->whereIn('SITEM', $sitemList);
        }

        return $result->get();
    }

    /**
     * 取得安全庫存
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @return \Illuminate\Support\Collection
     */
    public function GetSafetyStock(string $sDate, string $eDate, $designation = false)
    {
        $excludeItem = []; //欲排除料號

        //庫存子查詢
        $stockSub = DB::table('STOC')
            ->selectRaw('SITEM, SUM(SNUMB) AS SNUMB')
            ->whereIn('STCOD', ['A001', 'B000', 'B001', 'B002']) //出貨倉、待調撥倉、主倉儲、零售倉
            ->groupBy('SITEM');
        //採購單號子查詢
        $purchaseSub = "SELECT TOP(1) SCOD1 FROM SISE WHERE SICOD=S.SITEM AND SDAT1 BETWEEN '{$sDate}' AND '{$eDate}' ORDER BY SDAT1 DESC";
        //盤點子查詢
        $inventoryWorkSub = DB::table('InventoryWorkDetails')
            ->selectRaw('ROW_NUMBER() OVER(PARTITION BY ICODE ORDER BY UpdateDate DESC) AS ROWNUM')
            ->addSelect('ICODE', 'UpdateDate', 'Quantity');

        $result = DB::table(DB::raw("({$this->GetSqlWithParameter($stockSub)}) AS S"))
            ->select('I.IPNAM', 'I.ICODE', 'I.INAME', 'S.SNUMB', DB::raw('CONVERT(VARCHAR, IW.UpdateDate, 120) AS UpdateDate'), 'IW.Quantity')
            ->addSelect(DB::raw("({$purchaseSub}) AS SCOD1"), 'I4.T11_11')
            ->join('ITEM AS I', 'I.ICODE', 'S.SITEM')
            ->join('ITEM1 AS I1', function ($join) { //排除產品註記暫停出貨
                $join
                    ->on('I1.ICODE1', 'I.ICODE')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'I.ICODE')
                    ->where('I3.IPSTOP', 0);
            })
            ->leftJoinSub($inventoryWorkSub, 'IW', function ($join) {
                $join
                    ->on('IW.ICODE', 'S.SITEM')
                    ->where('IW.ROWNUM', 1);
            })
            ->leftJoin('ITEM4 AS I4', function ($join) {
                $join
                    ->on('I4.ICODE4', 'I.ICODE');
            })
            ->orderBy('S.SITEM');
        if ($designation) {
            $result->where('I4.T11_11', 'china');
        }
        //排除料號
        if (!empty($excludeItem)) {
            $result->whereNotIn('I.ICODE', $excludeItem);
        }

        return $result->get();
    }

    /**
     * 取得安全庫存紀錄
     *
     * @param string $date YYYY-MM-DD
     * @return \Illuminate\Support\Collection
     */
    public function GetSafetyStockHistory(string $date, $designation = false)
    {
        $result = SafetyStock::whereBetween('created_at', [$date . ' 00:00:00', $date . ' 23:59:59']);
        if ($designation) {
            $result->where('item_remark', 'china');
        }

        return $result->get();
    }

    /**
     * 取得所有庫存
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetStockAll()
    {
        $result = DB::table('STOC AS S')
            ->select('I.ICODE', 'I.INAME', 'S.STNAM', 'S.SNUMB')
            ->join('ITEM AS I', 'I.ICODE', 'S.SITEM')
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'I.ICODE')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'I.ICODE')
                    ->where('I3.IPSTOP', 0);
            })
            ->whereIn('S.STNAM', ['出貨倉', '主倉儲', '零售倉', '待調撥倉'])
            ->orderByDesc('S.STNAM');

        return $result->get();
    }

    /**
     * 取得庫存異常
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetStockUnusual()
    {
        $result = DB::table('STOC AS S')
            ->select('I.ICODE', 'I.INAME', 'S.STNAM', 'S.SNUMB')
            ->join('ITEM AS I', 'I.ICODE', 'S.SITEM')
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'I.ICODE')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'I.ICODE')
                    ->where('I3.IPSTOP', 0);
            })
            ->where('S.SNUMB', '<', 0)
            ->orderBy('I.ICODE');

        return $result->get();
    }

    /**
     * 取得庫存可銷售量
     *
     * @param string|null $id 料號
     * @return \Illuminate\Support\Collection
     */
    public function GetStockCanSale($id = null)
    {
        //庫存子查詢
        $stockSub = DB::table('STOC')
            ->selectRaw('SITEM, SUM(SNUMB) AS Stock')
            ->whereNotIn('STCOD', ['D001', 'I001']) //排除不良品倉、報廢倉
            ->groupBy('SITEM');
        if (!empty($id)) {
            $stockSub->where('SITEM', $id);
        }
        //訂單需求子查詢
        $orderSub = DB::table('ORDET AS M')
            ->selectRaw('D.OICOD, SUM(D.OINUM) AS OrderNeed')
            ->join('ORSUB AS D', 'D.OCOD1', 'M.OCOD1')
            ->leftJoin('POSEIN AS P', 'P.PCOD2', 'M.OCOD1')
            ->where('M.OCANC', '!=', '訂單取消')
            ->whereNull('P.PCOD2')
            ->whereNotNull('D.OINUM')
            ->groupBy('D.OICOD');
        if (!empty($id)) {
            $orderSub->where('D.OICOD', $id);
        }

        $result = DB::table(DB::raw("({$this->GetSqlWithParameter($stockSub)}) AS S"))
            ->select('S.SITEM', 'I.INAME', 'S.Stock', 'O.OrderNeed')
            ->leftJoinSub($orderSub, 'O', 'O.OICOD', 'S.SITEM')
            ->join('ITEM AS I', 'I.ICODE', 'S.SITEM')
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'I.ICODE')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'I.ICODE')
                    ->where('I3.IPSTOP', 0);
            })
            ->orderBy('S.SITEM');

        return $result->get();
    }

    /**
     * 取得儲位所有產品資料
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetStorageAll()
    {
        $result = DB::table('ITEMP AS P')
            ->select('P.IICOD', 'P.ICODE', 'I.INAME', 'S1.SNUMB', 'S2.SNUMB AS SNUMB_MAIN')
            ->join('ITEM AS I', function ($join) {
                $join
                    ->on('I.ICODE', 'P.ICODE')
                    ->where('I.ICODE', '!=', 'xxx123');
            })
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'I.ICODE')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'I.ICODE')
                    ->where('I3.IPSTOP', 0);
            })
            ->leftJoin('STOC AS S1', function ($join) {
                $join
                    ->on('S1.SITEM', 'P.ICODE')
                    ->where('S1.STNAM', '出貨倉');
            })
            ->leftJoin('STOC AS S2', function ($join) {
                $join
                    ->on('S2.SITEM', 'P.ICODE')
                    ->where('S2.STNAM', '主倉儲');
            })
            ->orderBy('P.IICOD')
            ->orderBy('P.ICODE');

        return $result->get();
    }

    /**
     * 取得產品總庫存資料
     *
     * @param array $icodeList 產品料號清單
     * @return \Illuminate\Support\Collection
     */
    public function GetItem(array $icodeList = null)
    {
        $result = DB::table('ITEM AS I')
            ->select('I.ICODE', 'I.IPNAM', 'I.INUMB')
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'I.ICODE')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'I.ICODE')
                    ->where('I3.IPSTOP', 0);
            });
        if (!empty($icodeList)) {
            $result->whereIn('I.ICODE', $icodeList);
        }
        return $result->get();
    }

    /**
     * 取得產品詳細資料
     *
     * @param array $icodeList 產品料號清單
     * @return \Illuminate\Support\Collection
     */
    public function GetItemDetail(array $icodeList)
    {
        $result = DB::table('ITEM AS I')
            ->select('I.ICODE', 'I.INAME', 'I.IUNIT')
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'I.ICODE')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'I.ICODE')
                    ->where('I3.IPSTOP', 0);
            })
            ->whereIn('I.ICODE', $icodeList);

        return $result->get();
    }

    /**
     * 取得產品儲位資料
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetItemp()
    {
        $result = DB::table('ITEMP')
            ->select('IICOD', 'ICODE')
            ->orderBy('IICOD')
            ->orderBy('ICODE');

        return $result->get();
    }

    /**
     * 取得指定料號的未進貨採購資料
     *
     * @param array $icodeList
     * @return \Illuminate\Support\Collection
     */
    public function GetPurchaseNotIn(array $icodeList)
    {
        if (empty($icodeList)) {
            return collect();
        }
        $result = DB::table('SISE')
            ->select('SICOD', 'SCOD1')
            ->where('SINYN', 'N') //Y=進貨完畢、N=未進貨完畢、C=取消
            ->whereIn('SICOD', $icodeList)
            ->orderByDesc('SDAT1');

        return $result->get();
    }


    /**
     * 取得指定日期的重物件數
     *
     * @param string $twDate 民國日期。yyy.mm.dd
     * @return mixed
     */
    public function GetSalesFloor(string $twDate)
    {
        $result = DB::table('POSEIN AS M')
            ->where('M.PDAT1', $twDate);

        $this->GetSalesWithWeight($result);

        return $result->sum('D.HINU3');
    }


    /**
     * 取得指定區間的銷貨單號
     *
     * @param boolean $isHeavy 是否為重物
     * @return array
     */
    public function GetSalesOrderNo(bool $isHeavy = false)
    {
        $result = DB::table('POSEIN AS M')
            ->select('M.PCOD2')
            ->whereNotIn('M.PPCOD', self::excludeedCustomers)
            ->distinct();
        if ($isHeavy) {
            $this->GetSalesWithWeight($result);
        }

        return $result->pluck('PCOD2')->toArray();
    }

    /**
     * 取得指定區間，列印次數>=1的銷貨資料，並排除有退貨
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @param boolean $isHeavy 是否為重物
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesPrinted(string $sDate, string $eDate, bool $isHeavy = false)
    {
        $result = DB::table('POSEIN AS M')
            ->select('M.PCOD1', 'M.PCOD2', 'M.PJONO', 'M.PPNAM', 'M.ConsignTran', 'M.PCMAN')
            ->addSelect('M.PTELE', 'M.PBAK1', 'M.PBAK2', 'M.PBAK3', 'M.InsideNote', 'L.CREATE_Date')
            ->leftJoin('PoseinCheckDataLog AS L', function ($join) {
                $join
                    ->on('L.PCOD1', 'M.PCOD1')
                    ->where('L.CheckCount', 1);
            })
            ->whereBetween('M.PDAT1', [$sDate, $eDate])
            ->where('M.PPRIN', '>=', 1)
            ->whereNotIn('M.PCOD2', function ($query) {
                $query
                    ->select('PCOD2')
                    ->from('POSEOU')
                    ->whereNotNull('PCOD2')
                    ->where('PCOD2', '!=', '')
                    ->distinct();
            })
            ->whereNotIn('M.PPCOD', self::excludeedCustomers)
            ->distinct('M.PCOD1')
            ->OrderBy('M.PCOD1');
        if ($isHeavy) {
            $this->GetSalesWithWeight($result);
        }

        return $result->get();
    }

    /**
     * 取得指定區間的銷售產品總數
     *
     * @param string $sDate 開始日期
     * @param string $eDate 結束日期
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesSum(string $sDate, string $eDate)
    {
        $result = DB::table('HISTIN')
            ->selectRaw('SUM(HINUM) as SUM,HICOD,HINAM')
            ->whereBetween('HDAT1', [$sDate, $eDate])
            ->whereNotIn('HICOD', ['XX123', 'XX124', 'XX126', 'xxx123', 'XXXYY001', 'XXYYZZ05', '99010022001', '99010022002', '99010022003', '99010022014', '11010002002'])
            ->whereNotBetween('HICOD', ['11010014001', '11010014024'])
            ->whereNotBetween('HICOD', ['11010020001', '11010020005'])
            ->groupby('HICOD', 'HINAM');

        return $result->get();
    }

    /**
     * 取得指定銷貨單的驗貨、託運紀錄
     *
     * @param string $id 銷貨單號/原始訂單號/備註1~3
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesCheck(string $id)
    {
        $checkDateQuery = 'SELECT TOP(1) CREATE_Date FROM PoseinCheckDataLog AS L WHERE L.PCOD1=P.PCOD1 ORDER BY CREATE_Date DESC';
        $transportNo = 'SELECT TOP(1) F.FNum FROM FreightNote AS F WHERE F.PCOD1=P.PCOD1 ORDER BY F.CREATE_Date DESC';
        $result = DB::table('POSEIN AS P')
            ->select(
                'P.PCOD1',
                'P.TransportCode',
                'P.TransportName',
                DB::raw("({$checkDateQuery}) AS CREATE_Date"),
                DB::raw("({$transportNo}) AS FNum"),
            )
            ->where('P.PCOD1', $id) //銷貨單號
            ->orWhere('P.PJONO', $id) //原始訂單號
            ->orWhere('P.PBAK1', 'LIKE', "%{$id}%") //備註1
            ->orWhere('P.PBAK2', 'LIKE', "%{$id}%") //備註2
            ->orWhere('P.PBAK3', 'LIKE', "%{$id}%") //備註3
            ->orderBy('P.PCOD1', 'DESC');

        return $result->get();
    }

    /**
     * 取得一個月內有銷貨、未驗貨銷貨單號
     *
     * @param string $sDate 民國起日。yyymmdd
     * @param string $eDate 民國訖日。yyymmdd
     * @return array
     */
    public function GetSalesUnchecked(string $sDate, string $eDate)
    {
        $result = DB::table('POSEIN AS P')
            ->select('P.PCOD1', 'P.PPCOD', 'P.PPNAM')
            ->leftJoin('PoseinCheckDataLog AS L', 'L.PCOD1', 'P.PCOD1')
            ->whereBetween('P.PDAT1', [$sDate, $eDate])
            ->whereNotIn('P.PPCOD', ['D001', 'x002', 'C001', 'C002']) //排除特定客戶。亞銘燈飾、半成品領料、面交、退換貨
            ->whereNull('L.CREATE_Date')
            ->orderBy('P.PCOD1');

        return $result->get();
    }

    /**
     * 取得指定時間內，未驗貨明細
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesUncheckedDetail(string $sDate, string $eDate)
    {
        $result = DB::table('HISTIN AS H')
            ->select('H.HCOD1', 'H.HICOD', 'H.HINUM')
            ->leftJoin('PoseinCheckDataLogDetails AS P', 'P.HCOD1', 'H.HCOD1')
            ->whereBetween('H.HDAT1', [$sDate, $eDate])
            ->whereNull('P.HCOD1')
            ->orderBy('H.HCOD1');

        return $result->get();
    }

    /**
     * 取得商品成長比較表
     *
     * @param string $a_sDate A區間民國起日。yyy.mm.dd
     * @param string $a_eDate A區間民國訖日。yyy.mm.dd
     * @param string $b_sDate B區間民國起日。yyy.mm.dd
     * @param string $b_eDate B區間民國訖日。yyy.mm.dd
     * @param string $id 指定料號
     * @param string $type 指定類別
     * @param string $customer 指定客戶代碼
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesComparison(string $a_sDate, string $a_eDate, string $b_sDate, string $b_eDate, string $id = null, string $type = null, string $customer = null)
    {
        $aSub = DB::table('HISTIN')
            ->select('HICOD', DB::raw('SUM(HTOTA) AS HTOTA'), DB::raw('SUM(HINUM) AS HINUM'))
            ->whereBetween('HDAT1', [$a_sDate, $a_eDate])
            ->groupBy('HICOD');

        $bSub = DB::table('HISTIN')
            ->select('HICOD', DB::raw('SUM(HTOTA) AS HTOTA'), DB::raw('SUM(HINUM) AS HINUM'))
            ->whereBetween('HDAT1', [$b_sDate, $b_eDate])
            ->groupBy('HICOD');

        if (!empty($id)) { //特定料號
            $aSub->where('HICOD', $id);
            $bSub->where('HICOD', $id);
        }
        if (!empty($type)) { //特定類別
            $aSub->where('HYCOD', $type);
            $bSub->where('HYCOD', $type);
        }
        if (!empty($customer)) { //特定客戶代碼
            $aSub->where('HPCOD', $customer);
            $bSub->where('HPCOD', $customer);
        }

        $result = DB::table(DB::raw("({$this->GetSqlWithParameter($aSub)}) AS A"))
            ->select('I.ICODE', 'I.INAME', 'I.ITNAM', 'A.HTOTA AS A_HTOTA', 'B.HTOTA AS B_HTOTA', 'A.HINUM AS A_HINUM', 'B.HINUM AS B_HINUM')
            ->joinSub($this->GetSqlWithParameter($bSub), 'B',  'B.HICOD', '=', 'A.HICOD', 'full')
            ->join('ITEM AS I', function ($join) {
                $join
                    ->on('I.ICODE', 'A.HICOD')
                    ->orOn('I.ICODE', 'B.HICOD');
            })
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'I.ICODE')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'I.ICODE')
                    ->where('I3.IPSTOP', 0);
            })
            ->orderByDesc('A_HTOTA');

        return $result->get();
    }

    /**
     * 取得商品成長月份比較表
     *
     * @param string $a_sDate A區間民國起日。yyy.mm.dd
     * @param string $a_eDate A區間民國訖日。yyy.mm.dd
     * @param string $b_sDate B區間民國起日。yyy.mm.dd
     * @param string $b_eDate B區間民國訖日。yyy.mm.dd
     * @param string $id 指定料號
     * @param string $type 指定類別
     * @param string $customer 指定客戶代碼
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesComparisonMonth(string $a_sDate, string $a_eDate, string $b_sDate, string $b_eDate, string $id = null, string $type = null, string $customer = null)
    {
        $aSub = DB::table('HISTIN AS D')
            ->select(DB::raw('SUBSTRING(D.HDAT1,5,2) AS HDAT1'), DB::raw('SUM(D.HTOTA) AS HTOTA'), DB::raw('SUM(D.HINUM) AS HINUM'))
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'D.HICOD')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'D.HICOD')
                    ->where('I3.IPSTOP', 0);
            })
            ->whereBetween('D.HDAT1', [$a_sDate, $a_eDate])
            ->groupBy(DB::raw('SUBSTRING(D.HDAT1,5,2)'));

        $bSub = DB::table('HISTIN AS D')
            ->select(DB::raw('SUBSTRING(D.HDAT1,5,2) AS HDAT1'), DB::raw('SUM(D.HTOTA) AS HTOTA'), DB::raw('SUM(D.HINUM) AS HINUM'))
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'D.HICOD')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'D.HICOD')
                    ->where('I3.IPSTOP', 0);
            })
            ->whereBetween('D.HDAT1', [$b_sDate, $b_eDate])
            ->groupBy(DB::raw('SUBSTRING(D.HDAT1,5,2)'));

        if (!empty($id)) { //特定料號
            $aSub->where('D.HICOD', $id);
            $bSub->where('D.HICOD', $id);
        }
        if (!empty($type)) { //特定類別
            $aSub->where('D.HYCOD', $type);
            $bSub->where('D.HYCOD', $type);
        }
        if (!empty($customer)) { //特定客戶代碼
            $aSub->where('D.HPCOD', $customer);
            $bSub->where('D.HPCOD', $customer);
        }

        $result = DB::table(DB::raw("({$this->GetSqlWithParameter($aSub)}) AS A"))
            ->select(DB::raw('CASE WHEN A.HDAT1 IS NULL THEN B.HDAT1 ELSE A.HDAT1 END AS HDAT1'))
            ->addSelect('A.HTOTA AS A_HTOTA', 'B.HTOTA AS B_HTOTA', 'A.HINUM AS A_HINUM', 'B.HINUM AS B_HINUM')
            ->joinSub($this->GetSqlWithParameter($bSub), 'B',  'B.HDAT1', '=', 'A.HDAT1', 'full')
            ->orderByDesc('A_HTOTA');

        return $result->get();
    }

    /**
     * 取得銷售產品類別
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesItemType()
    {
        $result = DB::table('ITYP')
            ->select('ITCOD', 'ITNAM')
            ->where('ITYPE', 2);

        return $result->get();
    }

    /**
     * 取得銷售明細
     *
     * @param string $id 銷貨單號
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesDetail(string $id)
    {
        $result = DB::table('POSEIN AS M')
            ->select('M.PCOD1', 'M.PCOD2', 'M.PJONO', 'M.PPNAM', 'M.PINTO', 'D.HICOD', 'D.HINAM', 'D.HUNIT', 'D.HINUM', 'D.HTOTA')
            ->join('HISTIN AS D', 'D.HCOD1', 'M.PCOD1')
            ->where('M.PCOD1', $id);

        return $result->get();
    }

    /**
     * 取得指定區間的客戶銷售額，未含稅
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesOfCustomer(string $sDate, string $eDate)
    {
        $result = DB::table('HISTIN')
            ->select('HPCOD', DB::raw('SUM(HTOTA) AS HTOTA'))
            ->whereBetween('HDAT1', [$sDate, $eDate])
            ->groupBy('HPCOD');

        return $result->get();
    }

    /**
     * 取得指定區間的商品銷售額
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesOfItem(string $sDate, string $eDate)
    {
        $result = DB::table('HISTIN AS H')
            ->select('HICOD', DB::raw('SUM(H.HTOTA) AS HTOTA'), DB::raw('SUM(H.HINUM*I.ISOUR) AS COST'), DB::raw('SUM(H.HINUM) AS HINUM'))
            ->join('ITEM AS I', 'I.ICODE', 'H.HICOD')
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'I.ICODE')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'I.ICODE')
                    ->where('I3.IPSTOP', 0);
            })
            ->whereBetween('H.HDAT1', [$sDate, $eDate])
            ->groupBy('H.HICOD');

        return $result->get();
    }


    /**
     * 取得指定區間的商品銷售額
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesDetailOfItem(string $sDate, string $eDate, array $itemList)
    {
        $result = DB::table('HISTIN')
            ->select('HICOD', 'HINUM', 'HDAT1')
            ->whereBetween('HDAT1', [$sDate, $eDate])
            ->whereIn('HICOD', $itemList)
            ->where('HINUM', '!=', 0);

        return $result->get();
    }

    /**
     * 取得指定區間的客戶銷售額，含稅
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @return mixed
     */
    public function GetSalesTotalTax(string $sDate, string $eDate)
    {
        $result = DB::table('POSEIN')
            ->select('PTOTA')
            ->whereBetween('PDAT1', [$sDate, $eDate]);

        return $result->sum('PTOTA');
    }

    /**
     * 取得地板拖運數量查詢
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesCount(string $sDate, string $eDate)
    {
        $result = DB::table('HISTIN AS D')
            ->select('D.HCOD1', 'D.HICOD', 'D.HINAM', 'D.HINUM')
            ->join('POSEIN AS M', 'M.PCOD1', 'D.HCOD1')
            ->whereBetween('D.HDAT1', [$sDate, $eDate])
            ->where('D.HYCOD', '11') //限定地板大類
            ->where('D.HPCOD', '!=', 'C001') //排除面交的客戶
            ->where('M.TransportCode', '!=', 'COM')
            ->orderBy('D.HICOD')
            ->orderBy('D.HCOD1'); //排除本公司親送

        return $result->get();
    }

    /**
     * 取得未有退貨的銷貨資料
     *
     * @param string $orderNo 銷貨單號
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesNotReturned(string $orderNo)
    {
        if (empty($orderNo)) {
            return null;
        } else {
            $result = DB::table('POSEIN AS M')
                ->select('D.HDAT1', 'D.HICOD', 'D.HINAM', 'D.HINUM')
                ->join('HISTIN AS D', 'D.HCOD1', 'M.PCOD1')
                ->where('M.PCOD1', $orderNo)
                ->whereNotIn('M.PCOD2', function ($query) {
                    $query
                        ->select('PCOD2')
                        ->from('POSEOU')
                        ->whereNotNull('PCOD2')
                        ->where('PCOD2', '!=', '')
                        ->distinct();
                })
                ->orderBy('D.HICOD');

            return $result->get();
        }
    }

    /**
     * 取得指定日期的每小時驗貨數量
     *
     * @param string $sDate 起日。yyyy-mm-dd HH:mm:ss
     * @param string $eDate 訖日。yyyy-mm-dd HH:mm:ss
     * @return \Illuminate\Support\Collection
     */
    public function GetPackageSpeed(string $sDate, string $eDate)
    {
        $result = DB::table('PoseinCheckDataLog')
            ->select('PCOD1', 'CREATE_Date')
            ->addSelect(DB::raw("(SUBSTRING(CONVERT(VARCHAR, CREATE_Date,108),1,2)+':00') AS CREATE_Hour"))
            ->where('CheckCount', 1)
            ->whereBetween('CREATE_Date', [$sDate, $eDate])
            ->orderBy('CREATE_Date');

        return $result->get();
    }

    /**
     * 取得滯銷商品表
     *
     * @param string $twDate 指定民國日期。yyy.mm.dd
     * @return \Illuminate\Support\Collection
     */
    public function GetUnsalable(string $twDate)
    {
        $purchaseSub = DB::table('VISEIN')
            ->select(DB::raw('ROW_NUMBER() OVER (PARTITION BY VICOD ORDER BY VDAT1 DESC) rownum, VICOD, VINAM, VINUM, VDAT1'))
            ->where('VDAT1', '<=', $twDate)
            ->whereNotIn('VYCOD', self::excludeedSalesType); //排除指定大類

        $result = DB::table(DB::raw("({$this->GetSqlWithParameter($purchaseSub)}) AS I"))
            ->select(DB::raw('I.VICOD, I.VINAM, I.VINUM, I.VDAT1, SUM(O.HINUM) AS HINUM'))
            ->join('ITEM AS S', function ($join) {
                $join
                    ->on('S.ICODE', 'I.VICOD')
                    ->where('S.INUMB', '!=', 0);
            })
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'S.ICODE')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'S.ICODE')
                    ->where('I3.IPSTOP', 0);
            })
            ->leftJoin('HISTIN AS O', function ($join) {
                $join
                    ->on('O.HICOD', 'I.VICOD')
                    ->on('O.HDAT1', '>=', 'I.VDAT1');
            })
            ->where('I.rownum', 1)
            ->groupBy('I.VICOD', 'I.VINAM', 'I.VINUM', 'I.VDAT1');

        return $result->get();
    }

    /**
     * 取得未列印
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @param boolean $isHeavy 是否為重物
     * @return \Illuminate\Support\Collection
     */
    public function GetNoPrint(string $sDate, string $eDate, bool $isHeavy = false)
    {
        $result = DB::table('POSEIN AS M')
            ->select('M.PCOD1', 'M.PCOD2', 'M.PJONO', 'M.PPNAM', 'M.ConsignTran', 'M.PCMAN')
            ->addSelect('M.PTELE', 'M.PBAK1', 'M.PBAK2', 'M.PBAK3', 'M.InsideNote')
            ->whereBetween('M.PDAT1', [$sDate, $eDate])
            ->where('M.PPRIN', 0)
            ->whereNotIn('M.PPCOD', self::excludeedCustomers)
            ->distinct('M.PCOD1');
        if ($isHeavy) {
            $this->GetSalesWithWeight($result);
        }
        return $result->get();
    }

    /**
     * 取得銷貨託運資料
     *
     * @param string $sDate 民國起日。yyymmdd
     * @param string $eDate 民國訖日。yyymmdd
     * @return \Illuminate\Support\Collection
     */
    public function GetTransport(string $sDate, string $eDate)
    {
        $subQuery = 'SELECT TOP(1) F.FNum FROM FreightNote AS F WHERE F.PCOD1=P.PCOD1 ORDER BY F.CREATE_DATE DESC';
        $result = DB::table('POSEIN AS P')
            ->select('P.TransportCode', 'P.TransportName', 'P.PCOD1', 'P.PPCOD', 'P.PPNAM')
            ->addSelect(DB::raw("({$subQuery}) AS FNum"))
            ->whereBetween('P.PDAT1', [$sDate, $eDate])
            ->where('P.PDAT1', '>=', '110.11.26')->whereNotIn('P.PCOD2', function ($query) {
                $query
                    ->select('PCOD2')
                    ->from('POSEOU')
                    ->whereNotNull('PCOD2')
                    ->where('PCOD2', '!=', '')
                    ->distinct();
            })
            ->orderBy('P.TransportCode')
            ->orderBy('P.PCOD1');

        return $result->get();
    }

    /**
     * 取得託運明細
     *
     * @param string $sDate 起日。yyyy/mm/dd hh:mm:ss
     * @param string $eDate 訖日。yyyy/mm/dd hh:mm:ss
     * @return \Illuminate\Support\Collection
     */
    public function GetTransportDetail(string $sDate, string $eDate)
    {
        $result =  DB::table('FreightNote AS F')
            ->select('F.FNum', 'F.PCOD1', 'P.PJONO', 'H.HICOD', 'H.HINAM', 'H.HINUM')
            ->join('HISTIN AS H', 'H.HCOD1', 'F.PCOD1')
            ->join('POSEIN AS P', 'P.PCOD1', 'F.PCOD1')
            ->where('F.FType', 'H')
            ->whereBetween('F.CREATE_DATE', [$sDate, $eDate]);

        return $result->get();
    }

    /**
     * 取得託運花費所需資料
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @return \Illuminate\Support\Collection
     */
    public function GetTransportCost(string $sDate, string $eDate)
    {
        $result = DB::table('POSEIN AS M')
            ->select('PCOD1', 'TransportCode', 'TransportName', 'PTOTA')
            ->whereBetween('PDAT1', [$sDate, $eDate])
            ->orderBy('PCOD1');

        return $result->get();
    }

    /**
     * 取得重物的訂單
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @return \Illuminate\Support\Collection
     */
    public function GetHeavyOrder(string $sDate, string $eDate)
    {
        $result = DB::table('ORSUB AS D')
            ->select('M.OCOD4', 'D.OCOD1', 'D.OINAM', 'D.OINUM', 'PI.PCOD1 AS PINID')
            ->join('ORDET AS M', function ($join) {
                $join
                    ->on('M.OCOD1', 'D.OCOD1')
                    ->where('M.TransportName', '!=', '新竹貨運')
                    ->where('M.OCANC', '!=', '訂單取消');
            })
            ->leftJoin('POSEIN AS PI', 'PI.PCOD2', 'D.OCOD1')
            ->leftJoin('POSEOU AS PO', 'PO.PCOD2', 'D.OCOD1')
            ->whereBetween('D.OICOD', ['20010023001', '20010023034']) //重物料號
            ->whereBetween('D.ODATE', [$sDate, $eDate])
            ->whereNull('PO.PCOD1')
            ->whereNotIn('D.OCCOD', self::excludeedCustomers)
            ->orderBy('D.OCOD1');

        return $result->get();
    }

    /**
     * 取得訂單的備註
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @return \Illuminate\Support\Collection
     */
    public function GetOrderRemarks(string $sDate, string $eDate)
    {
        $result = DB::table('ORDET AS O')
            ->select('O.OCOD1', 'O.OCOD4', 'O.OBAK1', 'O.OBAK2', 'O.OBAK3')
            ->leftJoin('POSEIN AS P', 'P.PCOD2', 'O.OCOD1')
            ->leftJoin('Sinvo AS S', 'S.Invono', 'P.PTXCO')
            ->whereBetween('O.ODATE', [$sDate, $eDate])
            ->where(function ($query) { //有備註資料
                $query
                    ->where('O.OBAK1', '!=', '')
                    ->orWhere('O.OBAK2', '!=', '')
                    ->orWhere('O.OBAK3', '!=', '');
            })
            ->where(function ($query) { //未壓統一編號
                $query
                    ->whereNull('S.Ctcod')
                    ->orWhere('S.Ctcod', '');
            })
            ->whereNotIn('O.OCCOD', self::excludeedCustomers)
            ->orderByDesc('O.OCOD1');

        return $result->get();
    }

    /**
     * 取得訂單詳細資料
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @param string $id 訂單單號
     * @return \Illuminate\Support\Collection
     */
    public function GetOrderDetail(string $sDate, string $eDate, string $id)
    {
        $result = DB::table('ORDET AS M')
            ->select('M.OCOD1', 'M.OCNAM', 'M.OCCOD', 'M.OTOT1', 'D.OICOD', 'D.OINAM', 'D.OUNIT', 'D.OINUM', 'D.OTOTA', 'D.OINYN')
            ->join('ORSUB AS D', 'D.OCOD1', 'M.OCOD1')
            ->where('M.ODAT2', '')
            ->where('M.OCANC', '!=', '訂單取消')
            ->whereBetween('M.ODATE', [$sDate, $eDate])
            ->where('M.OCOD1', $id);

        return $result->get();
    }

    /**
     * 取得訂單正貨資料
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @return \Illuminate\Support\Collection
     */
    public function GetOrderSample(string $sDate, string $eDate)
    {
        $transportNoSubQuery = 'SELECT TOP(1) F.FNum FROM FreightNote AS F WHERE F.PCOD1=P.PCOD1 ORDER BY F.CREATE_Date DESC';

        $result = DB::table('ORDET AS M')
            ->select('M.OCOD1', 'M.OCOD4', 'M.ODATE', 'M.OBAK1')
            ->addSelect('P.PCOD1', 'P.TraningStatusCode',  'FS.Fstatus')
            ->addSelect(DB::raw("({$transportNoSubQuery}) AS FNum"))
            ->addSelect('D.OICOD', 'D.OINAM', 'D.OINUM')
            ->join('ORSUB AS D', 'D.OCOD1', 'M.OCOD1')
            ->leftJoin('POSEIN AS P', 'P.PCOD2', 'M.OCOD1')
            ->leftJoin('FreightStatus AS FS', 'FS.FID', 'P.TraningStatusCode')
            ->whereBetween('M.ODATE', [$sDate, $eDate])
            ->whereIn('D.OICOD', self::floorCode)
            ->where('M.OCANC', '!=', '訂單取消') //排除訂單取消
            ->where('M.OCCOD', '!=', 'x002') //排除半成品領料
            ->orderBy('M.OCOD1');

        return $result->get();
    }

    /**
     * 取得訂單樣片資料
     *
     * @param array $idList
     * @return \Illuminate\Support\Collection
     */
    public function GetOrderSampleDetail(array $idList)
    {
        $transportNoSubQuery = 'SELECT TOP(1) F.FNum FROM FreightNote AS F WHERE F.PCOD1=P.PCOD1 ORDER BY F.CREATE_Date DESC';

        $result = DB::table('ORDET AS M')
            ->select('M.OCOD1', 'M.OBAK1', 'FS.Fstatus', 'P.PCOD1')
            ->addSelect(DB::raw("({$transportNoSubQuery}) AS FNum"))
            ->leftJoin('POSEIN AS P', 'P.PCOD2', 'M.OCOD1')
            ->leftJoin('FreightStatus AS FS', 'FS.FID', 'P.TraningStatusCode')
            ->whereIn('M.OCOD1', $idList)
            ->orderBy('M.OCOD1')
            ->distinct('M.OCOD1');

        return $result->get();
    }

    /**
     * 紀錄特殊訂單匯出紀錄
     *
     * @param string $sDate 起日
     * @param string $eDate 訖日
     * @param string $type 匯出類型
     * @param string $file 匯出內容JSON
     * @return void
     */
    public function CreateOrderSampleExportLog(string $sDate, string $eDate, string $type, string $file)
    {
        OrderSampleFile::create([
            'start_date' => $sDate,
            'end_date' => $eDate,
            'convert_type' => $type,
            'convert_file' => $file,
        ]);
    }

    /**
     * 取得指定原始訂單號的TMS訂單、銷貨資訊
     *
     * @param array $originalOrderList 原始訂單編號清單
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesByOriginalOrder(array $originalOrderList)
    {
        $result = DB::table('ORDET')
            ->select('OCOD4', 'OCOD1', 'PCOD1')
            ->leftJoin('POSEIN', 'PCOD2', 'OCOD1')
            ->whereIn('OCOD4', $originalOrderList);

        return $result->get();
    }

    /**
     * 取得訂單正貨資料
     *
     * @param array $idList
     * @return \Illuminate\Support\Collection
     */
    public function GetOrderGenuineDetail(array $idList)
    {
        $result = DB::table('ORSUB')
            ->select('OCOD1', 'OICOD', 'OINAM', 'OINUM')
            ->whereIn('OCOD1', $idList)
            ->whereIn('OICOD', self::floorCode);

        return $result->get();
    }

    /**
     * 取得需額外放贈品(感應燈)的訂單
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @return \Illuminate\Support\Collection
     */
    public function GetBulbOrder(string $sDate, string $eDate)
    {
        $result = DB::table('ORSUB AS D')
            ->select('D.OCOD1', 'D.OINUM', 'D.OICOD')
            ->join('ORDET AS M', 'M.OCOD1', 'D.OCOD1')
            ->whereBetween('D.ODATE', [$sDate, $eDate])
            ->whereNotIn('D.OCCOD', self::excludeedCustomers)
            ->where(function ($query) {
                //需要放贈品的料號
                $query->whereBetween('D.OICOD', ['10003005022', '10003005032']);
                $query->orwhere('D.OICOD', '10003005037');
                $query->orwhere('D.OICOD', '10003005038');
                //贈品料號
                $query->orWhereIn('D.OICOD', self::bulbCode);
            })->orderBy('D.OCOD1');

        return $result->get();
    }

    /**
     * 取得指定訂單單號，銷貨單內有放贈品的訂單單號
     *
     * @param array $idList 訂單單號清單
     * @return array
     */
    public function HasBulb(array $idList)
    {
        if (empty($idList)) {
            return array();
        }
        $result = DB::table('POSEIN AS M')
            ->select('M.PCOD2')
            ->join('HISTIN AS D', 'D.HCOD1', 'M.PCOD1')
            ->whereIn('M.PCOD2', $idList)
            ->whereIn('D.HICOD', self::bulbCode)
            ->distinct();

        return $result->pluck('PCOD2')->toArray();
    }

    /**
     * 取得地板產品
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetFloorItem()
    {
        $result = DB::table('ITEM')
            ->select('ICODE', 'INAME', 'IUNIT')
            ->where('ITCOD', '11')
            ->orWhereIn('ICODE', ['99010014001', '99010014002', '99010014003', '99010022004', 'XX123']);

        return $result->get();
    }

    /**
     * 取得產品履歷
     *
     * @param string $sDate 民國起日。yyy.mm.dd
     * @param string $eDate 民國訖日。yyy.mm.dd
     * @param string $id 產品料號
     * @return \Illuminate\Support\Collection
     */
    public function GetItemLog(string $sDate, string $eDate, string $id)
    {
        //進貨
        $purchase = DB::table('VISEIN AS D')
            ->select('D.VTIME AS Time', DB::raw("'進貨' AS Type"), 'D.VCOD1 AS Id', 'D.VTCOD AS StockCode', 'D.VTNAM AS Stock', 'D.VINUM AS Num')
            ->addSelect('M.BUSER AS UserName')
            ->leftJoin('BUYSIN AS M', 'M.BCOD1', 'D.VCOD1')
            ->whereBetween('D.VDAT1', [$sDate, $eDate])
            ->where('D.VICOD', $id);
        //進貨退回
        $rPurchase = DB::table('VISEOU AS D')
            ->select('D.VTIME AS Time', DB::raw("'進貨退回' AS Type"), 'D.VCOD1 AS Id', 'D.VTCOD AS StockCode', 'D.VTNAM AS Stock', 'D.VINUM AS Num')
            ->addSelect('M.BUSER AS UserName')
            ->leftJoin('BUYSOU AS M', 'M.BCOD1', 'D.VCOD1')
            ->whereBetween('D.VDAT1', [$sDate, $eDate])
            ->where('D.VICOD', $id);
        //銷貨
        $sales = DB::table('HISTIN AS D')
            ->select('D.HTIME AS Time', DB::raw("'銷貨' AS Type"), 'D.HCOD1 AS Id', 'D.HTCOD AS StockCode', 'D.HTNAM AS Stock', 'D.HINUM AS Num')
            ->addSelect('M.PUSER AS UserName')
            ->leftJoin('POSEIN AS M', 'M.PCOD1', 'D.HCOD1')
            ->whereBetween('D.HDAT1', [$sDate, $eDate])
            ->where('D.HICOD', $id);
        //銷貨退回
        $rSales = DB::table('HISTOU AS D')
            ->select('D.HTIME AS Time', DB::raw("'銷貨退回' AS Type"), 'D.HCOD1 AS Id', 'D.HTCOD AS StockCode', 'D.HTNAM AS Stock', 'D.HINUM AS Num')
            ->addSelect('M.PUSER AS UserName')
            ->leftJoin('POSEOU AS M', 'M.PCOD1', 'D.HCOD1')
            ->whereBetween('D.HDAT1', [$sDate, $eDate])
            ->where('D.HICOD', $id);
        //調撥
        $tran = DB::table('TRAN1 AS T')
            ->select('T.TTIME AS Time', DB::raw("'調撥' AS Type"), 'T.TCCOD AS Id', 'T.TSCO2 AS StockCode', DB::raw("(I1.ITNAM+'→'+I2.ITNAM) AS Stock"), 'T.TNUMB AS Num')
            ->addSelect('M.MUSER AS UserName')
            ->leftJoin('MTRAN AS M', 'M.MCODE', 'T.TCCOD')
            ->leftJoin('ITYP AS I1', 'I1.ITCOD', 'T.TSCO1')
            ->leftJoin('ITYP AS I2', 'I2.ITCOD', 'T.TSCO2')
            ->whereBetween('T.TDATE', [$sDate, $eDate])
            ->where('T.TICOD', $id);
        //盤點盈虧
        $inventory = DB::table('PDYM AS D')
            ->select('D.PTIME AS Time', 'D.PTNAM AS Type', 'D.PCODE AS Id', 'D.PSCOD AS StockCode', 'D.PSNAM AS Stock', 'D.PNUMB AS Num')
            ->addSelect('M.DNAME AS UserName')
            ->leftJoin('DAYM AS M', 'M.DCODE', 'D.PCODE')
            ->whereBetween('D.PDATE', [$sDate, $eDate])
            ->where('D.PICOD', $id);

        $result = $purchase
            ->union($rPurchase)
            ->union($sales)
            ->union($rSales)
            ->union($tran)
            ->union($inventory)
            ->orderBy('Time');

        return $result->get();
    }

    /**
     * 取得產品庫存紀錄
     *
     * @param string $sDate 西元起日。yyyy-mm-dd hh:mm:ss
     * @param string $eDate 西元訖日。yyyy-mm-dd hh:mm:ss
     * @param string $id 產品料號
     * @return \Illuminate\Support\Collection
     */
    public function GetItemHistory(string $sDate, string $eDate, string $id)
    {
        try {
            $result = DB::connection('mysql')
                ->table('log_item')
                ->select('ICODE', 'INUMB', DB::raw('DATE(created_at) AS created_at'))
                ->where('ICODE', $id)
                ->whereBetween('created_at', [$sDate, $eDate])
                ->orderBy('ICODE')
                ->orderBy('created_at')
                ->distinct('ICODE', 'created_at');

            return $result->get();
        } catch (Exception $ex) {
            return collect();
        }
    }

    /**
     * 取得所有產品料號
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetItemAll()
    {
        $result = DB::table('ITEM AS I')
            ->select('I.ICODE', 'I.INAME')
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'I.ICODE')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'I.ICODE')
                    ->where('I3.IPSTOP', 0);
            })
            ->orderBy('I.ICODE')
            ->distinct();

        return $result->get();
    }

    /**
     * 取得所有產品的成本
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetItemAllCost()
    {
        $result = DB::table('ITEM AS I')
            ->select('I.ICODE', 'I.INAME', 'I.ISOUR')
            ->join('ITEM1 AS I1', function ($join) { //排除暫停出貨的料號
                $join
                    ->on('I1.ICODE1', 'I.ICODE')
                    ->where('I1.ISTOP', 0);
            })
            ->join('ITEM3 AS I3', function ($join) { //排除產品註記廠商停產
                $join
                    ->on('I3.ICODE3', 'I.ICODE')
                    ->where('I3.IPSTOP', 0);
            })
            ->orderBy('I.ICODE')
            ->distinct();
        return $result->get();
    }

    /**
     * 取得廣告花費
     *
     * @param string $sDate yyyy/mm/dd
     * @param string $eDate yyyy/mm/dd
     * @return \Illuminate\Support\Collection
     */
    public function GetAdCost(string $sDate, string $eDate)
    {
        try {
            $result = DB::connection('mysql')
                ->table('ad_cost')
                ->whereBetween('src_date', [$sDate, $eDate])
                ->orderBy('src_date');

            return $result->get();
        } catch (Exception $ex) {
            return collect();
        }
    }

    /**
     * 取得參數
     *
     * @param string $kind
     * @return \Illuminate\Support\Collection
     */
    public function GetConfig(string $kind)
    {
        try {
            $result = DB::connection('mysql')
                ->table('sys_config')
                ->select('config')
                ->where('enabled', 'Y')
                ->where('kind', $kind);

            return $result->get();
        } catch (Exception $ex) {
            return collect();
        }
    }

    /**
     * 取得商品對應
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetAdItem()
    {
        try {
            return AdItem::all();
        } catch (Exception $ex) {
            return collect();
        }
    }

    /**
     * 貨運狀態
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetFreightStatus()
    {
        $result = DB::table('FreightStatus')
            ->whereNotIn('FID', ['-1', '0']);

        return $result->get();
    }

    /**
     * 取得客戶代碼
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetCustomer()
    {
        $result = DB::table('CUST')
            ->select('CCODE', 'CNAM2')
            ->distinct();

        return $result->get();
    }

    /**
     * 蝦皮商品設定匯入
     *
     * @param array $data
     * @return void
     */
    public function SetAdItem(array $data)
    {
        try {
            AdItem::query()->update(['shopee' => null, 'shopee_created_at' => null]);
            $created_at = new DateTime('now');
            foreach ($data as $id => $shopee) {
                AdItem::updateOrCreate(['id' => $id], ['id' => $id, 'shopee' => $shopee, 'shopee_created_at' => $created_at]);
            }
        } catch (Exception $ex) {
        }
    }

    /**
     * 是否有包驗貨排班
     *
     * @param string $settle_date 日期
     * @return boolean
     */
    public function HasPackageShift(string $settle_date)
    {
        $db = PackageShiftM::with('Details')->where('settle_date', $settle_date)->first();
        return !empty($db) && count($db->Details) > 0;
    }

    /**
     * 取得重物的銷貨資訊
     *
     * @param Builder $sql
     * @return Builder
     */
    private function GetSalesWithWeight(Builder $sql)
    {
        if (!empty($sql)) {
            //JOIN銷貨明細檔，並限定重物料號
            $sql->join('HISTIN AS D', function (Builder $join) {
                $join
                    ->on('D.HCOD1', 'M.PCOD1')
                    ->whereBetween('D.HICOD', ['11010014001', '11010014024']);
            });
        }
        return $sql;
    }

    /**
     * 取得帶參數的SQL語法
     *
     * @param Builder $builder
     * @return string
     */
    private function GetSqlWithParameter(Builder $builder)
    {
        if (empty($builder)) {
            return '';
        }
        $bindings = $builder->getBindings();
        $sql = str_replace('?', "'%s'", $builder->toSql());
        return sprintf($sql, ...$bindings);
    }
}
