<?php

namespace App\Services;

use DateTime;
use Exception;
use Carbon\Carbon;
use App\Models\Order;
use Carbon\CarbonPeriod;
use App\Models\LogShipment;
use Illuminate\Support\Str;
use App\Models\OrderSampleM;
use Illuminate\Http\Request;
use App\Models\OrderSampleFile;
use App\Models\ProductionDiary;
use App\Models\CustomerComplaint;
use App\Models\PoseinCheckDataLog;
use App\Repositories\BaseRepository;

class BaseService
{
    /**
     * 土方重量設定檔
     */
    public const EARTH_WEIGHT_CONFIG = [
        '黑火山' => 1000,
        '泥炭' => 3000,
        '赤玉' => 800,
        '煉石' => 500,
        '蛭石' => 200
    ];

    private $configService;
    private $baseRepository;

    public function __construct(BaseRepository $baseRepository, ConfigService $configService)
    {
        $this->baseRepository = $baseRepository;
        $this->configService = $configService;
    }

    /**
     * 取得民國日期格式。YYY.MM.DD
     *
     * @param DateTime $date
     * @return string
     */
    public function GetTWDateStr(DateTime $date): string
    {
        $tmp = isset($date) ?  clone $date : new DateTime('now');
        $tmp->modify('-1911 year');
        return ltrim($tmp->format('Y.m.d'), '0');
    }

    /**
     * 將民國日期格式轉成DateTime
     *
     * @param string $twDate 民國日期格式。YYY.MM.DD
     * @return DateTime
     */
    public function ConvertTWToDate(string $twDate)
    {
        try {
            $tmp = explode('.', $twDate);
            $y = $tmp[0] + 1911;
            $m = $tmp[1];
            $d = $tmp[2];
            return new DateTime("{$y}/{$m}/{$d} 00:00:00");
        } catch (\Throwable $th) {
            return null;
        }
    }

    /**
     * 取得Log日期格式。YYYY/MM/DD
     *
     * @param DateTime $date
     * @return string
     */
    public function GetLogDateStr(DateTime $date): string
    {
        $tmp = isset($date) ?  clone $date : new DateTime('now');
        return $tmp->format("Y/m/d");
    }

    /**
     * 取的忽略毫秒的時間字串
     *
     * @param string|null $time
     * @return string|null
     */
    public function GetTimeIgnoreMS(?string $time)
    {
        //驗貨時間
        if (!empty($time)) {
            return (new DateTime($time))->format('Y-m-d H:i:s');
        } else {
            return $time;
        }
    }

    /**
     * 取得訂單狀況。未覆核、未轉單、已過期
     *
     * @param Request $request
     * @return array
     */
    public function GetOrderAll(Request $request)
    {
        $isHeavy = $this->IsHeavy($request);
        //erp訂單日期格式
        $date = new DateTime("now");
        //今日
        $eDate = $this->GetTWDateStr($date);
        //今日往前15天
        $date->modify("-15 day");
        $sDate = $this->GetTWDateStr($date);

        $orderAllDB = $this->baseRepository->GetOrderAll($sDate, $eDate, $isHeavy);
        $salesOrderNoDB = $this->baseRepository->GetSalesOrderNo($isHeavy);

        $result = array();
        $orderNoList = array();
        foreach ($orderAllDB as $value) {
            $orderNo = (string)$value->OCOD1;
            $orderStatus = $value->OCANC;

            if ($orderStatus == '待核准') {
                $result['unchecked'][] = $value; //未覆核
            }
            if ($orderStatus == '已核示' && !in_array($orderNo, $salesOrderNoDB)) {
                $value->CanPrint = false; //初始化可轉而未轉欄位
                $result['unprinted'][] = $value; //未轉單
                if (!in_array($orderNo, $orderNoList)) {
                    $orderNoList[] = $orderNo;
                }
            }
            if (
                $orderStatus != '訂單取消' && !in_array($orderNo, $salesOrderNoDB) &&
                floor((strtotime(str_replace('.', '-', $eDate)) - strtotime(str_replace('.', '-', $value->ODATE))) / 86400) >= 2
            ) {
                $result['expired'][] = $value; //已過期
            }
        }

        //判斷未轉單當中可轉而未轉
        if (!empty($orderNoList)) {
            $canPrintOrder = array();
            $products = $this->baseRepository->GetOrderProduct($orderNoList);
            //取得未轉單的所有產品庫存
            $productIDList =  $products->pluck('OICOD')->toArray();
            $productIDList = array_unique($productIDList);
            $stock = $this->GetStock($productIDList);
            foreach ($products->groupBy('OCOD1') as $orderNo => $group) {
                $isSatisfied = true;
                foreach ($group as $value) {
                    $itemNo = (string)$value->OICOD;
                    $orderNeedNum = $value->OINUM;
                    $outStockNum = $stock[$itemNo]['出貨倉'];
                    if ($orderNeedNum > $outStockNum) {
                        $isSatisfied = false;
                        break;
                    }
                }
                if ($isSatisfied) {
                    $canPrintOrder[] = $orderNo;
                }
            }
            $result['canPrint'] = count($canPrintOrder);
            if (!empty($canPrintOrder)) {
                foreach ($result['unprinted'] as $value) {
                    $orderNo = (string)$value->OCOD1;
                    $value->CanPrint = in_array($orderNo, $canPrintOrder);
                }
            }
        }
        return $result;
    }

    /**
     * 取得面交訂單，排除已驗貨
     *
     * @param Request $request
     * @return array
     */
    public function GetOrderFaceToFace(Request $request)
    {
        $isHeavy = $this->IsHeavy($request);
        //erp訂單日期格式
        $date = new DateTime("now");
        //今日
        $eDate = $this->GetTWDateStr($date);
        //今日往前15天
        $date->modify("-15 day");
        $sDate = $this->GetTWDateStr($date);

        return $this->baseRepository->GetOrderFaceToFace($sDate, $eDate, $isHeavy)->toArray();
    }

    /**
     * 取得訂單詳細資料
     *
     * @param Request $request
     * @return \Illuminate\Support\Collection
     */
    public function GetOrderDetail(Request $request)
    {
        $id = $request->id;
        if (empty($id)) {
            return collect();
        }
        //erp訂單日期格式
        $date = new DateTime("now");
        //今日
        $eDate = $this->GetTWDateStr($date);
        //今日往前15天
        $date->modify("-15 day");
        $sDate = $this->GetTWDateStr($date);

        return  $this->baseRepository->GetOrderDetail($sDate, $eDate, $id);
    }

    /**
     * 取得未轉單總表
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetOrderUnprinted()
    {
        $db = $this->baseRepository->GetOrderUnprinted();
        $today = new DateTime((new DateTime('now'))->format('Y-m-d 00:00:00'));
        foreach ($db as $value) {
            if (empty($value->ODATE)) {
                continue;
            }
            $date = $this->ConvertTWToDate($value->ODATE);
            $elapsedDays = $today->diff($date)->days;

            $value->ElapsedDays = $elapsedDays;
            $value->Date = $date->format('Y/m/d');
            $value->IsToday = $elapsedDays == 0;
        }
        return $db;
    }

    /**
     * 取得儲位總表
     *
     * @return array
     */
    public function GetStorageAll()
    {
        $db = $this->baseRepository->GetStorageAll();
        if (empty($db)) {
            return collect();
        } else {
            return $db->groupBy('IICOD');
        }
    }

    /**
     * 取得安全庫存
     *
     * @param Request $request
     * @return \Illuminate\Support\Collection
     */
    public function GetSafetyStock(Request $request = null)
    {
        $designation = empty($request) ? false : $request->has('designation');
        //今日
        $eDate = $this->GetTWDateStr(Carbon::today());
        $aMonthAgo = $this->GetTWDateStr(Carbon::today()->addDays(-30));
        $twoMonthAgo = $this->GetTWDateStr(Carbon::today()->addDays(-60));
        $sixMonthAgo = $this->GetTWDateStr(Carbon::today()->addDays(-180));

        $safetyStockDB = $this->baseRepository->GetSafetyStock($twoMonthAgo, $eDate, $designation);
        $salesDetailDB = $this->baseRepository->GetSalesDetailOfItem($sixMonthAgo, $eDate, $safetyStockDB->unique('ICODE')->pluck('ICODE')->toArray());

        //計算個料號每日平均銷售
        $salesDetail = array();
        foreach ($salesDetailDB->groupBy('HICOD') as $item_no => $group) {
            //取30日平均->60日平均->180日平均
            $dailySales = round($group->whereBetween('HDAT1', [$aMonthAgo, $eDate])->sum('HINUM') / 30);
            if ($dailySales == 0) {
                $dailySales = round($group->whereBetween('HDAT1', [$twoMonthAgo, $eDate])->sum('HINUM') / 60);
                if ($dailySales == 0) {
                    $dailySales = round($group->whereBetween('HDAT1', [$sixMonthAgo, $eDate])->sum('HINUM') / 180);
                }
            }
            if ($dailySales > 0) {
                $salesDetail[$item_no] =  $dailySales;
            }
        }

        foreach ($safetyStockDB as $key => $value) {
            $item_no = $value->ICODE;
            //每日平均銷售
            if (array_key_exists($item_no, $salesDetail)) {
                $dailySales = $salesDetail[$item_no];
            } else {
                unset($safetyStockDB[$key]);
                continue;
            }

            //庫存剩餘天數
            if ($dailySales == 0) {
                $remainingDay = 0;
            } else {
                $remainingDay = round(($value->SNUMB) / $dailySales);
            }

            $value->DailySales = $dailySales; //平均每日銷售量
            $value->RemainingDay = $remainingDay; //庫存剩餘天數
            $value->SuggestBuy = $dailySales * 30; //建議採買量30天
        }

        return $safetyStockDB->sortBy('RemainingDay');
    }

    /**
     * 取得安全庫存紀錄
     *
     * @param Request $request
     * @return \Illuminate\Support\Collection
     */
    public function GetSafetyStockHistory(Request $request)
    {
        $designation = empty($request) ? false : $request->has('designation');
        $date = $request->get('date', Carbon::today()->format('Y-m-d'));
        return $this->baseRepository->GetSafetyStockHistory($date, $designation);
    }

    /**
     * 取得庫存總表
     *
     * @return array
     */
    public function GetStockAll()
    {
        $result = array();
        //整理產品庫存數量
        $dbStockAll = $this->baseRepository->GetStockAll();
        foreach ($dbStockAll as $value) {
            $itemID = (string)$value->ICODE;
            $itemName = $value->INAME;
            $stockName = $value->STNAM;
            $stockNum = number_format($value->SNUMB);

            if (!array_key_exists($itemID, $result)) {
                $result[$itemID]['Name'] = $itemName;
            }
            $result[$itemID]['Stock'][$stockName] = $stockNum;
        }
        //整理產品儲位
        $dbItemp = $this->baseRepository->GetItemp();
        foreach ($dbItemp as $value) {
            $itemID = (string)$value->ICODE;
            $storage = $value->IICOD;

            if (array_key_exists($itemID, $result)) {
                $result[$itemID]['Storage'][] = $storage;
            }
        }
        return $result;
    }

    /**
     * 取得搓合總表資料
     *
     * @return array
     */
    public function GetStockCheck()
    {
        //15天內所有訂單
        $order = $this->GetOrder();

        //取得料號清單
        $icodeList = array();
        foreach ($order as $itemNo => $value) {
            array_push($icodeList, (string)$itemNo);
        }
        $items = $this->baseRepository->GetItem($icodeList);
        $purchaseNotIn = $this->GetPurchaseNotIn($icodeList);

        $result = array();
        foreach ($order as $itemNo => $value) {
            foreach ($value as $itemName => $value2) {
                $item = $items->where('ICODE', $itemNo)->first();
                $itemStock = $item->INUMB ?? 0;
                $orderNoList = $value2['order'];
                $orderNeedNum = $value2['amount'];
                $purchase = array();
                if (array_key_exists($itemNo, $purchaseNotIn)) {
                    $purchase = $purchaseNotIn[$itemNo];
                }

                //料號
                $result[$itemNo]['no'] = $itemNo;
                //品名
                $result[$itemNo]['name'] = $itemName;
                //供應商
                $result[$itemNo]['supplier'] = $item->IPNAM ?? '';
                //訂單需求數
                $result[$itemNo]['demand'] = number_format($orderNeedNum);
                //總庫存數
                $result[$itemNo]['Stock'] = number_format($itemStock);
                //需採購數量/剩餘數
                $result[$itemNo]['rubbing'] = number_format($itemStock - $orderNeedNum);
                //未進貨採購單號
                $result[$itemNo]['purchase'] = $purchase;
                //需求訂單
                $result[$itemNo]['order'] = $orderNoList;
                //需求訂單數量
                $result[$itemNo]['orderCount'] = count($orderNoList);
            }
        }

        foreach ($result as $key => $value) {
            if ($value['rubbing'] >= 0) {
                unset($result[$key]);
            }
        }

        return  $result;
    }

    /**
     * 取得庫存異常
     *
     * @return \Illuminate\Support\Collection
     */
    public function GetStockUnusual()
    {
        return $this->baseRepository->GetStockUnusual();
    }

    /**
     * 取得庫存可銷售量
     *
     * @param Request $request
     * @return array
     */
    public function GetStockCanSale(Request $request)
    {
        //erp訂單日期格式
        $date = new DateTime("now");
        //今日
        $eDate = $this->GetTWDateStr($date);
        //今日往前15天
        $date->modify("-15 day");
        $sDate = $this->GetTWDateStr($date);

        $db = $this->baseRepository->GetStockCanSale($request->id);
        $uncheckedDb = $this->baseRepository->GetSalesUncheckedDetail($sDate, $eDate);
        $itemList = $this->baseRepository->GetItemAll();

        foreach ($db as $item) {
            $item->CanSale = $item->Stock - $item->OrderNeed;
            $item->UncheckedList = $uncheckedDb->where('HICOD', $item->SITEM);
        }
        return ['data' => $db, 'itemList' => $itemList];
    }

    /**
     * 取得需調撥總表資料
     *
     * @return array
     */
    public function GetTransferData()
    {
        $order = $this->GetOrder();

        //取得料號清單
        $icodeList = array();
        foreach ($order as $itemNo => $value) {
            array_push($icodeList, (string)$itemNo);
        }
        $stock = $this->GetStock($icodeList);

        $result = array();
        foreach ($order as $itemNo => $value) {
            foreach ($value as $itemName => $value2) {
                $orderNo = implode(',', array_keys($value2['order']));
                $orderNeedNum = $value2['amount'];
                $totalStockNum = $stock[$itemNo]['total'];
                $outStockNum = $stock[$itemNo]['出貨倉'];

                //料號
                $result[$itemNo]['no'] = $itemNo;
                //品名
                $result[$itemNo]['name'] = $itemName;
                //訂單需求數
                $result[$itemNo]['demand'] = number_format($orderNeedNum);
                //總庫存
                $result[$itemNo]['result'] = number_format($totalStockNum);
                //出貨倉不足數
                $result[$itemNo]['rubbing'] = intval($outStockNum) - intval($orderNeedNum);
                //需求訂單
                $result[$itemNo]['order'] = $orderNo;
                //庫存明細
                $result[$itemNo]['STNAM'] = $stock[$itemNo];
            }
        }

        foreach ($result as $key => $value) {
            if ($value['STNAM']['total'] < $value['demand']  || $value['rubbing'] >= 0) {
                unset($result[$key]);
            }
        }

        return  $result;
    }

    /**
     * 取得今日重物件數
     *
     * @return mixed
     */
    public function GetSalesFloor()
    {
        $date = new DateTime("now");
        $twDate = $this->GetTWDateStr($date);
        return $this->baseRepository->GetSalesFloor($twDate);
    }

    /**
     * 取得儲位調整資料
     *
     * @return array
     */
    public function GetSalesReserve()
    {
        $lv1Data = array();
        $lv2Data = array();
        $itempData = array();

        //erp訂單日期格式
        $date = new DateTime("now");
        //今日
        $eDate = $this->GetTWDateStr($date);
        //今日往前30天
        $date->modify("-30 day");
        $sDate = $this->GetTWDateStr($date);

        //取得指定區間下，產品平均銷售量，並標示級距
        $salesSumDB = $this->baseRepository->GetSalesSum($sDate, $eDate);
        $salesData = array();
        foreach ($salesSumDB as $value) {
            $id = $value->HICOD;
            $sum = $value->SUM;
            //只取有值的
            if (!empty($sum) && $sum != 0) {
                $salesData[$id] = $sum / 30;
            }
        }
        arsort($salesData); //排序平均每日出貨量，由大到小
        $lv1Avg = array_sum($salesData) / count($salesData); //第一次平均數
        foreach ($salesData as $key => $value) {
            //低於第一次平均數的都不要
            if ($value <= $lv1Avg) {
                unset($salesData[$key]);
            }
        }
        $lv2Avg = array_sum($salesData) / count($salesData); //第二次平均數
        foreach ($salesData as $key => $value) {
            //第一級距=數量>=第二次平均數*2
            //第二級距=第二次平均數<=數量<第二次平均數*2
            if ($value >= floor($lv2Avg * 2)) {
                $lv1Data[$key] = $value;
            } else if ($value >= floor($lv2Avg)) {
                $lv2Data[$key] = $value;
            }
        }

        //取得儲位產品
        $itempDB = $this->baseRepository->GetItemp();
        foreach ($itempDB as $value) {
            $itempData[$value->IICOD][] = $value->ICODE;
        }
        ksort($itempData);

        return
            [
                'lv1Data' => $lv1Data,
                'lv2Data' => $lv2Data,
                'itempData' => $itempData
            ];
    }

    /**
     * 取得指定銷貨單的驗貨、託運紀錄
     *
     * @param Request $request
     * @return array
     */
    public function GetSalesCheck(Request $request)
    {
        $result = array();
        $id = $request->id; //銷貨單號/原始訂單號/備註1~3
        if (empty($id)) {
            return $result;
        }
        $db = $this->baseRepository->GetSalesCheck($id);
        if (!empty($db)) {
            foreach ($db as $value) {
                $pcod1 = $value->PCOD1;
                $transportCode = $value->TransportCode;
                $transportName = $value->TransportName;
                $checkDate = $value->CREATE_Date;
                $transportID = $value->FNum;
                //取得託運類別的包裹查詢網站
                switch ($transportCode) {
                    case 'Shop-7':
                        $shopUrl = 'https://eservice.7-11.com.tw/e-tracking/search.aspx';
                        break;
                    case 'Shop-F':
                        $shopUrl = 'https://www.famiport.com.tw/Web_Famiport/page/process.aspx';
                        break;
                    case 'Shop-L':
                        $shopUrl = 'http://www.hilife.com.tw/serviceInfo_search.aspx';
                        break;
                    case 'Shop-OK':
                        $shopUrl = 'https://ecservice.okmart.com.tw/Tracking/Search';
                        break;
                    default:
                        $shopUrl = '';
                        break;
                }
                $result[$pcod1]['TransportName'] = $transportName;
                $result[$pcod1]['ShopURL'] = $shopUrl;
                $result[$pcod1]['CheckDate'] = $this->GetTimeIgnoreMS($checkDate); //驗貨時間
                $result[$pcod1]['TransportID'] = $transportID; //託運單號
            }
        }
        return $result;
    }

    /**
     * 取得未驗貨、當日已出貨
     *
     * @param Request $request
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesCheckStatus(Request $request)
    {
        $isHeavy = $this->IsHeavy($request);
        //erp訂單日期格式
        $date = new DateTime("now");
        $today = $date->format('Y-m-d'); //今天日期
        $twEDate = $this->GetTWDateStr($date); //今天民國日期
        $date->modify("-15 day");       //今日往前15天
        $twSDate = $this->GetTWDateStr($date);

        $salesDB = $this->baseRepository->GetSalesPrinted($twSDate, $twEDate, $isHeavy);

        foreach ($salesDB as $value) {
            if (empty($value->CREATE_Date)) {
                $value->CheckStatus = 1; //未有驗貨紀錄
            } else if (substr($value->CREATE_Date, 0, 10) == $today) {
                $value->CheckStatus = 2; //今日出貨
            } else {
                $value->CheckStatus = 0; //非今日出貨
            }
        }

        return $salesDB;
    }

    /**
     * 取得商品成長比較表
     *
     * @param Request $request
     * @return array
     */
    public function GetSalesComparison(Request $request)
    {
        $result = array();

        $group_type = $request->get('group_type', '');
        $id = $request->get('id', null);
        $type = $request->get('type', null);
        $customer = $request->get('customer', null);

        switch ($group_type) {
            case 'month':
                $a_year = $request->get('a_year', Carbon::today()->format('Y'));
                $b_year = $request->get('b_year', Carbon::today()->format('Y'));
                $a_start_date = $this->GetTWDateStr(new DateTime($a_year . '-01-01'));
                $a_end_date = $this->GetTWDateStr(new DateTime($a_year . '-12-31'));
                $b_start_date = $this->GetTWDateStr(new DateTime($b_year . '-01-01'));
                $b_end_date = $this->GetTWDateStr(new DateTime($b_year . '-12-31'));

                $db =  $this->baseRepository->GetSalesComparisonMonth($a_start_date, $a_end_date, $b_start_date, $b_end_date, $id, $type, $customer);
                foreach ($db as $value) {
                    $month = intval($value->HDAT1);
                    $aTotal = floatval($value->A_HTOTA);
                    $bTotal = floatval($value->B_HTOTA);
                    $aNum = floatval($value->A_HINUM);
                    $bNum = floatval($value->B_HINUM);
                    $diff = $aTotal - $bTotal; //A-B
                    $diffNum = $aNum - $bNum;
                    if (empty($bTotal)) {
                        $diffPercent = '';
                    } else {
                        $diffPercent = number_format(($diff / $bTotal) * 100) . '%'; //(A-B)/B*100%
                    }
                    if (empty($bNum)) {
                        $diffNumPercent = '';
                    } else {
                        $diffNumPercent = number_format(($diffNum / $bNum) * 100) . '%'; //(A-B)/B*100%
                    }

                    $result[$month]['aTotal'] = number_format($aTotal);
                    $result[$month]['aNum'] = number_format($aNum);
                    $result[$month]['bTotal'] = number_format($bTotal);
                    $result[$month]['bNum'] = number_format($bNum);
                    $result[$month]['diff'] = number_format($diff);
                    $result[$month]['diffPercent'] = $diffPercent;
                    $result[$month]['diffNum'] = number_format($diffNum);
                    $result[$month]['diffNumPercent'] = $diffNumPercent;
                }
                break;
            default:
                $a_start_date = $this->GetTWDateStr(new DateTime($request->a_start_date));
                $a_end_date = $this->GetTWDateStr(new DateTime($request->a_end_date));
                $b_start_date = $this->GetTWDateStr(new DateTime($request->b_start_date));
                $b_end_date = $this->GetTWDateStr(new DateTime($request->b_end_date));

                $db =  $this->baseRepository->GetSalesComparison($a_start_date, $a_end_date, $b_start_date, $b_end_date, $id, $type, $customer);
                foreach ($db as $value) {
                    $itemNo = (string)$value->ICODE;
                    $itemName = $value->INAME;
                    $itemType = $value->ITNAM;
                    $aTotal = floatval($value->A_HTOTA);
                    $bTotal = floatval($value->B_HTOTA);
                    $aNum = floatval($value->A_HINUM);
                    $bNum = floatval($value->B_HINUM);
                    $diff = $aTotal - $bTotal; //A-B
                    $diffNum = $aNum - $bNum;
                    if (empty($bTotal)) {
                        $diffPercent = '';
                    } else {
                        $diffPercent = number_format(($diff / $bTotal) * 100) . '%'; //(A-B)/B*100%
                    }
                    if (empty($bNum)) {
                        $diffNumPercent = '';
                    } else {
                        $diffNumPercent = number_format(($diffNum / $bNum) * 100) . '%'; //(A-B)/B*100%
                    }

                    $result[$itemNo]['name'] = $itemName;
                    $result[$itemNo]['type'] = $itemType;
                    $result[$itemNo]['aTotal'] = number_format($aTotal);
                    $result[$itemNo]['aNum'] = number_format($aNum);
                    $result[$itemNo]['bTotal'] = number_format($bTotal);
                    $result[$itemNo]['bNum'] = number_format($bNum);
                    $result[$itemNo]['diff'] = number_format($diff);
                    $result[$itemNo]['diffPercent'] = $diffPercent;
                    $result[$itemNo]['diffNum'] = number_format($diffNum);
                    $result[$itemNo]['diffNumPercent'] = $diffNumPercent;
                }
                break;
        }

        $aRange = $a_start_date . '-' . $a_end_date;
        $bRange = $b_start_date . '-' . $b_end_date;
        $itemList = $this->baseRepository->GetItemAll();
        $typeList = $this->baseRepository->GetSalesItemType();
        $customerList = $this->baseRepository->GetCustomer();

        return [
            'groupType' => $group_type,
            'data' => $result,
            'itemList' => $itemList,
            'typeList' => $typeList,
            'customerList' => $customerList,
            'aRange' => $aRange,
            'bRange' => $bRange,
        ];
    }

    /**
     * 取得銷貨明細
     *
     * @param Request $request
     * @return \Illuminate\Support\Collection
     */
    public function GetSalesDetail(Request $request)
    {
        $id = $request->id;
        if (empty($id)) {
            return collect();
        } else {
            return $this->baseRepository->GetSalesDetail($id);
        }
    }

    /**
     * 取得指定區間驗貨完成速度
     *
     * @param Request $request
     * @return array
     */
    public function GetPackageSpeed(Request $request)
    {
        $result = array();
        $result['chart'] = ['x' => ['x'], 'y' => [''], 'title' => ''];
        //取得指定日期並轉換格式
        $a_date = new DateTime($request->a_date);
        $a_sTime = $this->GetLogDateStr($a_date) . ' 08:30:00';
        $a_eTime = $this->GetLogDateStr($a_date) . ' 18:30:00';
        $b_date = new DateTime($request->b_date);
        $b_sTime = $this->GetLogDateStr($b_date) . ' 08:30:00';
        $b_eTime = $this->GetLogDateStr($b_date) . ' 18:30:00';

        $xAxis = $this->GetEachHour($a_sTime, $a_eTime);
        $yAxis = array();
        //A日期
        $aDB = $this->baseRepository->GetPackageSpeed($a_sTime, $a_eTime);
        $aData[] = $a_date->format('Y-m-d');
        foreach ($xAxis as $value) {
            $target = $aDB->filter(function ($item) use ($value) {
                return $value == $item->CREATE_Hour;
            });
            $y = $target->count();
            array_push($aData, $y);
        }
        $yAxis[] = $aData;
        //B日期，且與A日期不同
        if ($b_sTime != $a_sTime && $b_eTime != $a_eTime) {
            $bDB = $this->baseRepository->GetPackageSpeed($b_sTime, $b_eTime);
            $bData[] = $b_date->format('Y-m-d');
            foreach ($xAxis as $value) {
                $target = $bDB->filter(function ($item) use ($value) {
                    return $value == $item->CREATE_Hour;
                });
                $y = $target->count();
                array_push($bData, $y);
            }
            $yAxis[] = $bData;
            $title = $a_date->format('Y-m-d') . ' 與 ' . $b_date->format('Y-m-d') . '包裝速度比較圖';
        } else {
            $title = $a_date->format('Y-m-d') . '包裝速度圖';
        }

        $result['chart'] = [
            'x' => $xAxis,
            'y' => $yAxis,
            'title' => $title
        ];

        return $result;
    }

    /**
     * 取得現在鐘頭、上個鐘頭驗貨完成速度
     *
     * @return array
     */
    public function GetNowPackageSpeed()
    {
        $nowTime = new DateTime("now");

        $db = $this->baseRepository->GetPackageSpeed($nowTime->format('Y-m-d 08:30:00'), $nowTime->format('Y-m-d 18:30:00'));
        $now = clone $nowTime; //現在時間
        $nowTime->modify('-1 hour');
        $lastHour = clone $nowTime; //1個小時前
        $nowTime->modify('-1 hour');
        $lastTwoHour = clone $nowTime; //2個小時前

        $nowHourData = $db->filter(function ($item) use ($lastHour, $now) {
            $time = new DateTime($item->CREATE_Date);
            return $lastHour < $time && $time <= $now;
        });
        $lastHourData = $db->filter(function ($item) use ($lastTwoHour, $lastHour) {
            $time = new DateTime($item->CREATE_Date);
            return $lastTwoHour < $time && $time <= $lastHour;
        });

        return [
            'now' => $nowHourData->count(),
            'last' => $lastHourData->count()
        ];
    }

    /**
     * 取得滯銷商品表
     *
     * @param Request $request
     * @return array
     */
    public function GetUnsalable(Request $request)
    {
        $result = array();
        if (empty($request->start_day)) {
            $startDay = 30;
        } else {
            $startDay = intval($request->start_day);
        }
        if (empty($request->standard)) {
            $standard = 30;
        } else {
            $standard = floatval($request->standard);
        }
        define('time', "-{$startDay} day"); //距今N天前，預設30
        $date = new DateTime("now");
        $date->modify(time);
        $twDate = $this->GetTWDateStr($date);

        $db = $this->baseRepository->GetUnsalable($twDate);
        foreach ($db as $value) {
            $itemNo = $value->VICOD;
            $itemName = $value->VINAM;
            $purchaseDate = $value->VDAT1;
            $purchaseNum = $value->VINUM;
            $salesNum = $value->HINUM;
            $salesPercentage = round(($salesNum / $purchaseNum) * 100, 2);

            //銷售量 <= 進貨量的N%
            if ($salesNum <= $purchaseNum * ($standard / 100)) {
                $result[] = [
                    'itemNo' => $itemNo,
                    'itemName' => $itemName,
                    'purchaseDate' => $purchaseDate,
                    'purchaseNum' => number_format($purchaseNum),
                    'salesNum' => number_format($salesNum),
                    'salesPercentage' => $salesPercentage
                ];
            }
        }

        return $result;
    }

    /**
     * 取得未列印
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Support\Collection
     */
    public function GetNoPrint(Request $request)
    {
        $isHeavy = $this->IsHeavy($request);
        //erp訂單日期格式
        $date = new DateTime("now");
        //今日
        $eDate = $this->GetTWDateStr($date);
        //今日往前15天
        $date->modify("-15 day");
        $sDate = $this->GetTWDateStr($date);

        return $this->baseRepository->GetNoPrint($sDate, $eDate, $isHeavy);
    }

    /**
     * 取得銷貨託運資料
     *
     * @param Request $request
     * @return array
     */
    public function GetTransport(Request $request)
    {
        $result = array();
        $twDate = $this->GetTWDateStr(new DateTime($request->date));
        //過去七天
        $wkDate = new DateTime($request->date);
        $wkDate->modify('-7 days');
        $wkDate = $this->GetTWDateStr($wkDate);
        $db = $this->baseRepository->GetTransport($wkDate, $twDate);
        $shipmentOrder = LogShipment::select('PCOD1')->whereBetween('PDATE', [$wkDate, $twDate])->distinct()->pluck('PCOD1')->toArray();
        //轉換格式
        foreach ($db as $value) {
            $isDone = in_array($value->PCOD1, $shipmentOrder); //有無上車紀錄

            //如果沒有上車紀錄則為空
            $value->isDone = $isDone;
            //託運類別代碼
            $tCode = $value->TransportCode;
            if (empty($tCode)) {
                $tCode = 'Unknow';
                $value->TransportName = '未知';
            }

            if (empty($result[$tCode])) {
                $result[$tCode]['All'] = 1;
                $result[$tCode]['Done'] = $isDone ? 1 : 0;
                $result[$tCode]['Data'][] = $value;
            } else {
                $result[$tCode]['All'] += 1;
                $result[$tCode]['Done'] += ($isDone ? 1 : 0);
                array_push($result[$tCode]['Data'], $value);
            }
        }
        $result = collect($result)->sortBy('All')->reverse()->toArray();

        return $result;
    }

    /**
     * 取得託運明細
     *
     * @param Request $request
     * @return array
     */
    public function GetTransportDetail(Request $request)
    {
        //取得指定日期並轉換格式
        $sDate = $request->start_date;
        if (empty($sDate)) {
            $tmp = new DateTime("now");
        } else {
            $tmp = new DateTime($sDate);
        }
        $sDate = $this->GetLogDateStr($tmp) . ' 00:00:00';
        $eDate = $request->end_date;
        if (empty($eDate)) {
            $tmp = new DateTime("now");
        } else {
            $tmp = new DateTime($eDate);
        }
        $eDate = $this->GetLogDateStr($tmp) . ' 23:59:59';

        $db = $this->baseRepository->GetTransportDetail($sDate, $eDate);
        $result = array();
        foreach ($db->groupBy('FNum') as $transportNo => $value) {
            $result[$transportNo] = $value;
        }
        return $result;
    }

    /**
     * 取得託運花費
     *
     * @param Request $request
     * @return array
     */
    public function GetTransportCost(Request $request)
    {
        //取得指定日期並轉換格式
        $sDate = $request->start_date;
        if (empty($sDate)) {
            $tmp = new DateTime("now");
            $tmp = new DateTime($tmp->format('Y-m-1 00:00:00'));
        } else {
            $tmp = new DateTime($sDate);
        }
        $sDate = $this->GetTWDateStr($tmp);
        $eDate = $request->end_date;
        if (empty($eDate)) {
            $tmp = new DateTime("now");
        } else {
            $tmp = new DateTime($eDate);
        }
        $eDate = $this->GetTWDateStr($tmp);

        $db = $this->baseRepository->GetTransportCost($sDate, $eDate);
        $salesTotal = $db->sum('PTOTA'); //總銷售額
        //計算運費，並排除非指定託運資料
        foreach ($db as $key => $value) {
            $cost = 0;
            switch ($value->TransportCode) {
                case 'HCT': //新竹貨運
                    $cost = 0;
                    break;
                case 'Shop-7': //超取(7-11)
                case 'Shop-F': //超取(全家)
                    $cost = 60;
                    break;
                default:
                    unset($db[$key]);
                    continue 2;
            }
            $value->Cost = $cost;
        }

        $budget = $salesTotal * 0.04;
        $cost = $db->sum('Cost');
        $remaining = $budget - $cost;

        return [
            'SalesTotal' => $salesTotal,
            'Budget' => $budget,
            'Cost' => $cost,
            'Remaining' => $remaining,
            'data' => $db
        ];
    }

    /**
     * 取得上車檢驗資料
     *
     * @param Request $request
     * @return array
     */
    public function GetShipmentCheck(Request $request)
    {
        $status = 0;
        $msg = '';
        $data = collect();

        $orderNo = $request->get('id');
        if (empty($orderNo)) {
            return [
                'status' => 0,
                'msg' => '請使用手持式掃描機驗貨',
                'data' => collect()
            ];
        }
        $data = $this->baseRepository->GetSalesNotReturned($orderNo);
        if (!empty($data) && count($data) > 0) {
            if (LogShipment::where('PCOD1', $orderNo)->count() > 0) {
                $status = 2;
                $msg = '已經上車過，請確認是否重複出貨';
            } else {
                $status = 3;
                $msg = '上車';
                LogShipment::Create(
                    [
                        'PDATE' => $data->first()->HDAT1,
                        'PCOD1' => $orderNo
                    ]
                );
            }
        } else {
            $status = 1;
            $msg = '不存在，或是已有退貨單，請勿出貨';
        }

        return [
            'status' => $status,
            'msg' => $msg,
            'orderNo' => $orderNo,
            'data' => $data
        ];
    }

    /**
     * 取得孤兒單總表
     *
     * @return array
     */
    public function GetSalesLostCheck()
    {
        //erp訂單日期格式
        $date = new DateTime("now");
        //今日
        $eDate = $this->GetTWDateStr($date);
        //今日往前15天
        $date->modify("-30 day");
        $sDate = $this->GetTWDateStr($date);

        //排除特定客戶。亞銘燈飾、半成品領料、面交、退換貨
        return $this->baseRepository->GetSalesUnchecked($sDate, $eDate);
    }

    /**
     * 取得地板拖運數量查詢
     *
     * @param Request $request
     * @return array
     */
    public function GetSalesCount(Request $request)
    {
        //取得指定日期並轉換格式
        $sDate = $this->GetTWDateStr(new DateTime($request->start_date));
        $eDate = $this->GetTWDateStr(new DateTime($request->end_date));
        $db = $this->baseRepository->GetSalesCount($sDate, $eDate);
        $result = array();
        foreach ($db->groupBy('HICOD') as $itemNo => $group) {
            $itemName = $group->first()->HINAM;

            $result[$itemNo] = [
                'Name' => $itemName,
                'Count' => $group->sum('HINUM'),
                'SalesNo' => $group->pluck('HCOD1')->toArray(),
            ];
        }

        return $result;
    }

    /**
     * 取得過重的訂單
     *
     * @return array
     */
    public function GetOverweightOrder()
    {
        //erp訂單日期格式
        $date = new DateTime("now");
        //今日
        $eDate = $this->GetTWDateStr($date);
        //今日往前15天
        $date->modify("-15 day");
        $sDate = $this->GetTWDateStr($date);

        $db = $this->baseRepository->GetHeavyOrder($sDate, $eDate);
        //取得訂單的總重量
        $result = array();
        foreach ($db as $value) {
            $itemID = (string)$value->OCOD1;
            $itemName = $value->OINAM;
            $itemNum = $value->OINUM;
            $orignID = (string)$value->OCOD4;
            $salesID = (string)$value->PINID;

            //初始化
            if (empty($result[$itemID]['Weight'])) {
                $result[$itemID]['Weight'] = 0;
            }
            $result[$itemID]['OriginalOrderNo'] = $orignID;
            $result[$itemID]['SalesNo'] = $salesID;

            $itemData = &$result[$itemID]['Weight'];
            $cfg = array_filter(self::EARTH_WEIGHT_CONFIG, function ($k) use ($itemName) {
                return strstr($itemName, $k);
            }, ARRAY_FILTER_USE_KEY);
            if (!empty($cfg)) {
                $itemData += ($itemNum * array_values($cfg)[0]);
            }
        }
        //取得過重的訂單
        if (!empty($result)) {
            foreach ($result as $key => $value) {
                //總重<=4500代表正常
                if ($value['Weight'] <= 4500) {
                    unset($result[$key]);
                }
            }
        }

        return $result;
    }

    /**
     * 取得訂單含有統編需求
     *
     * @param Request $request
     * @return array
     */
    public function GetOrderTaxID(Request $request)
    {
        if (empty($request->start_date) || empty($request->end_date)) {
            $nowTWDate = $this->GetTWDateStr(new DateTime("now"));
            $sDate = $nowTWDate;
            $eDate = $nowTWDate;
        } else {
            $sDate = $this->GetTWDateStr(new DateTime($request->start_date));
            $eDate = $this->GetTWDateStr(new DateTime($request->end_date));
        }
        $db = $this->baseRepository->GetOrderRemarks($sDate, $eDate);
        $result = array();
        if (!empty($db)) {
            foreach ($db as $value) {
                $orderNo = (string)$value->OCOD1;
                $remark1 = $value->OBAK1;
                $remark2 = $value->OBAK2;
                $remark3 = $value->OBAK3;
                if ($this->IsContainsTaxID([$remark1, $remark2, $remark3])) {
                    $result[$orderNo] = $value;
                }
            }
        }
        return $result;
    }

    /**
     * 取得訂單正貨樣片資料
     *
     * @return array
     */
    public function GetOrderSample()
    {
        $result = array();
        //erp訂單日期格式
        $date = new DateTime("now");
        //今日
        $eDate = $this->GetTWDateStr($date);
        //今日往前15天
        $date->modify("-30 day");
        $sDate = $this->GetTWDateStr($date);

        $freightStatus = $this->baseRepository->GetFreightStatus(); //取得貨運狀態
        $db = $this->baseRepository->GetOrderSample($sDate, $eDate);
        //解析訂單是否備註樣片單號
        $sampleIdList = array();
        foreach ($db as $value) {
            $bak = $value->OBAK1; //備註
            $start = mb_strpos($bak, '樣片#', 0, 'utf-8');
            if ($start === false) { //未含'樣片#'字樣
                $start = mb_strpos($bak, '樣卡#', 0, 'utf-8');
                if ($start === false) { //未含'樣片#'字樣
                    continue;
                }
            }
            $id = trim(mb_substr($bak, $start + 3, 11, 'utf-8'));
            if (!empty($id) && strlen($id) == 11 && is_numeric($id)) { //非空白、長度11、數值
                $value->sampleOrderNo = $id;
                if (!in_array($id, $sampleIdList)) {
                    $sampleIdList[] = $id;
                }
            }
        }
        $sampleDetailDb = $this->baseRepository->GetOrderSampleDetail($sampleIdList);
        $detailDb = $this->baseRepository->GetOrderGenuineDetail($db->unique('OCOD1')->pluck('OCOD1')->toArray());
        $today = new DateTime((new DateTime('now'))->format('Y-m-d 00:00:00'));
        foreach ($db->groupBy('OCOD1') as $orderNo => $group) {
            $value = $group->first();
            $orignalOrderNo = $value->OCOD4; //原始訂單號
            $salesNo = $value->PCOD1; //銷貨單號
            $transportNo = $value->FNum; //託運單號
            $transportStatus = $value->Fstatus; //託運狀態
            $transportStatusCode = $value->TraningStatusCode; //託運狀態代碼
            $isTransportRemark = false;
            $transportRemarkCode = $this->GetFreightStatus($value->OBAK1); //備註的托運狀態
            if (!empty($transportRemarkCode)) { //有備註託運狀態，以此判斷
                $isTransportRemark = true;
                $transportStatusCode = $transportRemarkCode;
                $transportStatus = $freightStatus->firstWhere('FID', $transportRemarkCode)->Fstatus ?? $transportRemarkCode;
            }
            //距今天數
            $date = $this->ConvertTWToDate($value->ODATE); //訂單日期
            $elapsedDay = $today->diff($date)->days;

            //樣片單
            $sampleOrderNo = $value->sampleOrderNo ?? null; //樣片訂單號
            $sampleSalesNo = '';
            $sampleTransportNo = ''; //樣片託運單號
            $sampleTransportStatus = ''; //樣片託運狀態
            $sampleChecked = false; //樣片確認狀態
            if (!empty($sampleOrderNo)) {
                $sampleData = $sampleDetailDb->firstWhere('OCOD1', $sampleOrderNo);
                if (!empty($sampleData)) {
                    $sampleSalesNo = $sampleData->PCOD1; //樣片銷貨單號
                    $sampleTransportNo = $sampleData->FNum; //樣片託運單號
                    $sampleTransportStatus = $sampleData->Fstatus; //樣片託運狀態
                    $isSampleTransportRemark = false;
                    $sampleTransportRemark = $this->GetFreightStatus($sampleData->OBAK1); //樣片備註的托運狀態
                    if (!empty($sampleTransportRemark)) {
                        $isSampleTransportRemark = true;
                        $sampleTransportStatus = $freightStatus->firstWhere('FID', $sampleTransportRemark)->Fstatus ?? $sampleTransportRemark;
                    }
                    $sampleChecked = mb_strpos($sampleData->OBAK1, '樣片已確認', 0, 'utf-8') !== false || mb_strpos($sampleData->OBAK1, '樣卡已確認', 0, 'utf-8') !== false;
                }
            }

            //狀態
            $status = null;
            $statusValue = null;
            if (!empty($salesNo) && !empty($sampleSalesNo) && !$sampleChecked) {
                $status = '錯誤';
                $statusValue = 'error';
            } else if (empty($sampleOrderNo) || $sampleChecked) { //不須樣片OR樣片已確認
                //正貨狀態
                if (empty($transportNo) && empty($transportStatusCode)) {
                    $status = '正貨可出貨';
                    $statusValue = 'canSales';
                } elseif (!empty($transportNo) && empty($transportStatusCode)) {
                    $status = '正貨已備貨';
                    $statusValue = 'prepared';
                } elseif ((!empty($transportNo) || $isTransportRemark) && in_array($transportStatusCode, ['1', '2', '3'])) { //已取件/配送中/重新配送
                    $status = '正貨出貨中';
                    $statusValue = 'transporting';
                } elseif ((!empty($transportNo) || $isTransportRemark) && $transportStatusCode == '5') { //已送達
                    $status = '正貨已送達';
                    $statusValue = 'arrived';
                }
            } else { //需樣片
                $elapsedDay = '';
                if (empty($sampleTransportNo) && empty($sampleTransportStatus)) {
                    $status = '需樣片';
                    $statusValue = 'needSample';
                } elseif ((!empty($sampleTransportNo) || $isSampleTransportRemark) && !empty($sampleTransportStatus)) {
                    $status = '樣片待回覆';
                    $statusValue = 'waitSample';
                }
            }

            //片數計算
            $packageNum = 0;
            $detail = $detailDb->where('OCOD1', $orderNo);
            if (count($detail) > 0) {
                foreach ($detail as $d) {
                    switch ($d->OICOD) {
                        case '11010014005':
                        case '11010014006':
                        case '11010014007':
                        case '11010014008':
                            $packageNum += ($d->OINUM * 10); //單位=1包=10片
                            $d->Package = round($d->OINUM, 2);
                            break;
                        case '11010014015':
                        case '11010014020':
                        case '11010014021':
                        case '11010014022':
                        case '11010014024':
                            $packageNum += $d->OINUM; //單位=1片
                            $d->Package = round($d->OINUM / 10, 2);
                            break;

                        default:
                            break;
                    }
                }
            }

            $result[] = [
                'orderNo' => $orderNo,
                'orignalOrderNo' => $orignalOrderNo,
                'salesNo' => $salesNo,
                'transportNo' => $transportNo,
                'transportStatus' => $transportStatus,
                'isTransportRemark' => $isTransportRemark,
                'sampleOrderNo' => $sampleOrderNo,
                'sampleSalesNo' => $sampleSalesNo,
                'sampleTransportNo' => $sampleTransportNo ?? '',
                'sampleTransportStatus' => $sampleTransportStatus ?? '',
                'isSampleTransportRemark' => $isSampleTransportRemark ?? false,
                'elapsedDay' => $elapsedDay,
                'packageNum' => round($packageNum / 10, 2), //換算包數
                'packageList' => $detail,
                'status' => $status ?? '未定義',
                'statusValue' => $statusValue ?? 'unknow'
            ];
        }

        //狀態分組，並自訂排序
        $statusList = array();
        if (!empty($result)) {
            $order = ['needSample', 'waitSample', 'canSales', 'prepared', 'transporting', 'arrived', 'error', 'unknow'];
            $tmp = collect($result)->sort(function ($a, $b) use ($order) {
                $pos_a = array_search($a['statusValue'], $order);
                $pos_b = array_search($b['statusValue'], $order);
                return $pos_a - $pos_b;
            })->groupBy('statusValue');
            foreach ($tmp as $statusValue => $group) { //遍歷各狀態裡面的訂單
                $statusData = array();
                foreach ($group as $g) { //遍歷訂單裡面的資訊
                    foreach ($g['packageList'] as $detail) { //遍歷資訊裡面的各商品細項
                        $itemNo = $detail->OICOD; //產品料號
                        $itemName = $detail->OINAM; //產品名稱
                        $itemNum = $detail->OINUM; //產品數量
                        $itemPackage = $detail->Package; //產品包數

                        if (array_key_exists($itemNo, $statusData)) {
                            $statusData[$itemNo]['Num'] += $itemNum;
                            $statusData[$itemNo]['Package'] += $itemPackage;
                        } else {
                            $statusData[$itemNo] = [
                                'Name' => $itemName,
                                'Num' => $itemNum,
                                'Package' => $itemPackage,
                            ];
                        }
                    }
                }
                ksort($statusData); //依料號排序
                //加入資料
                $statusList[$statusValue] = [
                    'name' => $group->first()['status'], //狀態名稱
                    'totalPackage' => collect($statusData)->sum('Package'), //該狀態總包數
                    'data' => $statusData, //狀態細項資料
                ];
            }
        }
        return [
            'data' => $result,
            'statusList' => collect($statusList),
            'freightStatus' => $freightStatus,
        ];
    }

    /**
     * 取得指定標頭的欄位索引
     *
     * @param array $header 標頭陣列資料
     * @return array
     */
    public function GetOrderSampleColumns(array $header)
    {
        if (empty($header)) {
            return null;
        }

        $result = [
            'order_date' => array_search('訂單成立日期', $header),
            'order_no' => array_search('訂單編號', $header),
            'item_total' => array_search('商品總價 (單)', $header),
            'buyer_shipping_fee' => array_search('買家支付的運費 (單)', $header),
            'shopee_shipping_fee' => array_search('蝦皮回饋運費', $header),
            'order_total' => array_search('訂單總金額 (單)', $header),
            'discount' => array_search('賣家折扣券折抵金額 (單)', $header),
            'item_name' => array_search('商品名稱 (品)', $header),
            'item_option_name' => array_search('商品選項名稱 (品)', $header),
            'item_price' => array_search('商品原價 (品)', $header),
            'item_discount' => array_search('商品活動價格 (品)', $header),
            'item_no' => array_search('商品選項貨號', $header),
            'item_quantity' => array_search('數量', $header),
            'customer_remark' => array_search('買家備註 (單)', $header),
            'remark' => array_search('備註 (單)', $header),
            'receiver_name' => array_search('收件者姓名 (單)', $header),
            'receiver_tel' => array_search('收件者電話 (單)', $header),
            'receiver_city' => array_search('城市 (單)', $header),
            'receiver_district' => array_search('行政區 (單)', $header),
            'receiver_postal' => array_search('郵遞區號 (單)', $header),
            'receiver_address' => array_search('收件地址 (單)', $header),
        ];

        foreach ($result as $r) {
            if ($r === false) {
                return null;
            }
        }

        return $result;
    }

    /**
     * 取得特殊訂單轉檔畫面資料
     *
     * @return array
     */
    public function GetOrderSampleConvertView()
    {
        $data = $this->GetOrderSampleConvert();
        $data = collect($data)->where('order_type', '!=', 0)->toArray();
        $floorItems = $this->baseRepository->GetFloorItem();

        return ['data' => $data, 'floorItems' => $floorItems];
    }

    /**
     * 取得已轉檔的樣片訂單
     *
     * @return array
     */
    public function GetOrderSampleConvert(string $sDate = null, string $eDate = null)
    {
        $result = array();
        $db = OrderSampleM::with('Edited')->with('EditedItem');
        if (!empty($sDate) && !empty($eDate)) {
            $db->whereBetween('order_date', [$sDate, $eDate]);
        }
        $db = $db->get();
        //該訂單編號，已有銷貨單號，進行排除
        $sales = $this->baseRepository->GetSalesByOriginalOrder($db->pluck('order_no')->unique()->toArray());
        foreach ($db as $key => $value) {
            $sale =  $sales->firstWhere('OCOD4', $value->order_no);
            //TMS內有訂單資訊 且 有銷貨單號
            if (!empty($sale) && !empty($sale->PCOD1)) {
                unset($db[$key]);
            }
        }
        //取得編輯後的商品資訊
        $editedItemNoList = array();
        foreach ($db as $item) {
            $editedItemNoList = array_merge($editedItemNoList, $item->EditedItem()->pluck('item_no')->toArray());
        }
        $editedItemNoList =  array_unique($editedItemNoList);
        $editedItemDetail = $this->baseRepository->GetItemDetail($editedItemNoList);
        //取得樣片設定檔
        $config = $this->configService->GetConfig('sample_item');
        if (!empty($config)) {
            //找出產品名稱
            $configValues = collect($config)->pluck('value')->toArray();
            $sampleItemDetail = $this->baseRepository->GetItemDetail($configValues);
            if (!empty($sampleItemDetail)) {
                foreach ($config as $cfg) {
                    $item = $sampleItemDetail->firstWhere('ICODE', $cfg->value);
                    $cfg->text = $item->INAME ?? '';
                    $cfg->price = $item->IUNIT ?? '';
                }
            }
        }
        //取得標題欄位設定
        $headerData = $this->configService->GetConfig('order_convert_columns');
        $columns = $this->GetOrderSampleColumns($headerData);
        if (empty($columns)) {
            return [];
        }
        //遍歷所有訂單
        foreach ($db as $order) {
            $orderArray = $order->toArray();
            $orderArray['original_data'] = $order->original_data; //toArray()不會解密資料，需手動解密並覆蓋
            $originalData = json_decode($order->original_data);
            if (empty($originalData)) { //無標頭資料不能正確分析
                continue;
            }
            //取第一筆原始資料的備註分析
            $firstRow = $originalData[0];
            if (count($firstRow) != count($headerData)) { //資料與標題欄位長度不一致
                continue;
            }
            $customerRemark = $firstRow[$columns['customer_remark']];
            $remark = $firstRow[$columns['remark']];
            if (!empty($remark) || !empty($customerRemark)) { //特殊訂單。買家/賣家有備註
                $isSampleOrder = false; //該訂單編號是否為樣片訂單
                $orderEdited = $order->Edited()->first(); //編輯的基本資料
                $orderEditedItem = $order->EditedItem()->get(); //編輯的商品項目
                $orderArray['customer_remark'] = $customerRemark;
                $orderArray['item_total'] = $firstRow[$columns['item_total']];
                $orderArray['receiver_name'] = $orderEdited->receiver_name ?? $firstRow[$columns['receiver_name']];
                $orderArray['receiver_tel'] = $orderEdited->receiver_tel ?? $firstRow[$columns['receiver_tel']];
                $orderArray['receiver_city'] = $orderEdited->receiver_city ?? $firstRow[$columns['receiver_city']];
                $orderArray['receiver_district'] = $orderEdited->receiver_district ?? $firstRow[$columns['receiver_district']];
                $orderArray['receiver_postal'] = $orderEdited->receiver_postal ?? $firstRow[$columns['receiver_postal']];
                $orderArray['receiver_address'] = $orderEdited->receiver_address ?? $firstRow[$columns['receiver_address']];
                //正貨商品明細，用於編輯正貨商品
                $genuine_item = array();
                if (!empty($orderEditedItem) && $orderEditedItem->count() > 0) { //有編輯，取編輯資料
                    foreach ($orderEditedItem as $item) {
                        $detail = $editedItemDetail->firstWhere('ICODE', $item->item_no);
                        $genuine_item[] = [
                            'item_no' =>  $item->item_no,
                            'item_name' => $detail->INAME ?? '', //TMS的商品名稱
                            'item_price' => $item->item_price,
                            'item_quantity' => $item->item_quantity,
                        ];
                    }
                } else { //無編輯，取原資料
                    foreach ($originalData as $item) {
                        $genuine_item[] = [
                            'item_no' =>  $item[$columns['item_no']],
                            'item_name' => $item[$columns['item_name']],
                            'item_price' => $item[$columns['item_price']],
                            'item_quantity' => $item[$columns['item_quantity']],
                        ];
                    }
                }
                $orderArray['genuine_item'] = json_encode($genuine_item);
                //判斷是否為樣片
                if (!empty($config) && !empty($remark)) {
                    $configKeys = collect($config)->pluck('key')->toArray();
                    foreach ($config as $cfg) {
                        $sampleItemKey = $cfg->key; //樣片關鍵字
                        $sampleItemNo = $cfg->value; //樣片料號
                        $sampleItemName = $cfg->text; //樣片名稱
                        //備註當中有樣片關鍵字 且 TMS有樣片料號
                        if (Str::contains($remark, $sampleItemKey)) {
                            $isSampleOrder = true;
                            $orderArray['order_no'] = $order->order_no . '-S';
                            $orderArray['item_no'] = $sampleItemNo;
                            $orderArray['item_name'] = $sampleItemName;
                            $orderArray['remark'] = ($orderEdited->remark ?? str_replace($configKeys, '', $remark)) . ($order->confirmed ? '樣片已確認' : ''); //樣片已確認則在備註加入關鍵字
                            $orderArray['order_type'] = 1;
                            $result[] = $orderArray;
                        }
                    }
                }
                if (!$isSampleOrder) { //非樣片單
                    $orderArray['remark'] = $orderEdited->remark ?? $remark;
                    $orderArray['order_type'] = 2;
                    foreach ($genuine_item as $item) {
                        $orderArray['item_no'] = $item['item_no'];
                        $orderArray['item_name'] = $item['item_name'];
                        $result[] = $orderArray;
                    }
                }
            } else { //正常訂單
                $orderArray['order_type'] = 0;
                $result[] = $orderArray;
            }
        }

        return $result;
    }

    /**
     * 取的特殊訂單匯出資料
     *
     * @param string $sDate 起日
     * @param string $eDate 訖日
     * @param string $type 類型。1:樣片已轉換、2:樣片已確認
     * @return array
     */
    public function GetOrderSampleExport(string $sDate, string $eDate, string $type)
    {
        $result = array();
        $orderSampleConvert = collect($this->GetOrderSampleConvert($sDate, $eDate));
        if (empty($orderSampleConvert) || $orderSampleConvert->count() == 0) {
            $this->baseRepository->CreateOrderSampleExportLog($sDate, $eDate, $type, json_encode($result));
            return $result;
        }
        $headerData = $this->configService->GetConfig('order_convert_columns');
        $result[] = $headerData;
        $columns = $this->GetOrderSampleColumns($headerData);

        switch ($type) {
            case '1': //樣片已轉換
                //以訂單編號分組
                foreach ($orderSampleConvert->groupBy('order_no') as $group) {
                    $firstOrder = $group->first();
                    $originalData = json_decode($firstOrder['original_data']);
                    if ($firstOrder['order_type'] == 1) { //樣片單需轉換部分欄位
                        //修改訂單單頭，取第一筆原始資料作為共通欄位的參考
                        $firstOriginalData = $originalData[0];
                        $firstOriginalData[$columns['order_no']] = $firstOrder['order_no']; //訂單編號加上"-S"
                        $firstOriginalData[$columns['order_total']] = '0'; //訂單總金額(單)=0
                        $firstOriginalData[$columns['item_total']] = '0'; //商品總價(單)=0
                        $firstOriginalData[$columns['item_price']] = '0'; //商品原價 (品)=0
                        $firstOriginalData[$columns['item_discount']] = '0'; //商品活動價格 (品)=0
                        $firstOriginalData[$columns['item_quantity']] = '1'; //數量=1
                        $firstOriginalData[$columns['buyer_shipping_fee']] = '0'; //買家支付的運費 (單)=0
                        $firstOriginalData[$columns['shopee_shipping_fee']] = '0'; //蝦皮回饋運費=0
                        $firstOriginalData[$columns['remark']] = $firstOrder['remark']; //備註
                        //修改訂單單身
                        foreach ($group as $g) {
                            $firstOriginalData[$columns['item_no']] = $g['item_no']; //商品選項貨號=樣片料號
                            $firstOriginalData[$columns['item_name']] = $g['item_name']; //商品名稱(品)=樣片名稱
                            $firstOriginalData[$columns['item_option_name']] = $g['item_name']; //商品選項名稱(品)=樣片名稱
                            $result[] = $firstOriginalData;
                        }
                    } elseif ($firstOrder['order_type'] != 2) { //正常訂單
                        $result = array_merge($result, $originalData);
                    }
                }
                break;
            case '2': //樣片已確認
                //是漾片單且樣片已確認，以訂單編號分組
                foreach ($orderSampleConvert->where('order_type', '!=', 0)->where('confirmed', 1)->groupBy('order_no') as $group) {
                    $firstOrder = $group->first();
                    $originalData = json_decode($firstOrder['original_data']);
                    $genuine_item = json_decode($firstOrder['genuine_item']);
                    //修改訂單單頭，取第一筆原始資料作為共通欄位的參考
                    $firstOriginalData = $originalData[0];
                    $firstOriginalData[$columns['receiver_name']] = $firstOrder['receiver_name']; //收件者姓名
                    $firstOriginalData[$columns['receiver_tel']] = $firstOrder['receiver_tel']; //收件者電話
                    $firstOriginalData[$columns['receiver_city']] = $firstOrder['receiver_city']; //收件者城市
                    $firstOriginalData[$columns['receiver_district']] = $firstOrder['receiver_district']; //收件者行政區
                    $firstOriginalData[$columns['receiver_postal']] = $firstOrder['receiver_postal']; //收件者由區號
                    $firstOriginalData[$columns['receiver_address']] = $firstOrder['receiver_address']; //收件者地址
                    $firstOriginalData[$columns['remark']] = $firstOrder['remark']; //備註
                    //修改訂單單身
                    foreach ($genuine_item as $g) {
                        $firstOriginalData[$columns['item_no']] = $g->item_no; //商品選項貨號=正貨料號
                        $firstOriginalData[$columns['item_name']] = $g->item_name; //商品名稱(品)=正貨名稱
                        $firstOriginalData[$columns['item_option_name']] = $g->item_name; //商品選項名稱(品)=正貨名稱
                        $firstOriginalData[$columns['item_price']] = $g->item_price; //商品原價 (品)=正貨售價
                        $firstOriginalData[$columns['item_discount']] = $g->item_price; //商品活動價格 (品)=正貨售價
                        $firstOriginalData[$columns['item_quantity']] = $g->item_quantity; //商品數量=正貨數量
                        $result[] = $firstOriginalData;
                    }
                }
                break;
        }

        //配件包處裡
        $acc_config = $this->configService->GetConfig('accessories_item'); //配件料號
        if (!empty($acc_config) && count($acc_config) > 0) {
            $acc_config = $this->baseRepository->GetItemDetail($acc_config);
        }
        $config = $this->configService->GetConfig('item_need_accessories'); //需配件的料號清單
        if (!empty($config) && !empty($acc_config)) {
            //略過單頭，依訂單編號分組，
            foreach (collect($result)->slice(1)->groupBy($columns['order_no']) as $orderNo => $group) {
                //判斷訂單購買內容，是否有需要放置配件包
                $needAccessories = $group->filter(function ($item) use ($columns, $config) {
                    $isNeed = false;
                    foreach ($config as $cfg) {
                        $item_quantity = $cfg->key;
                        $item_no = $cfg->value;
                        //指地的料號 且 數量達到，放配件包
                        if (in_array($item[$columns['item_no']], $item_no) && $item[$columns['item_quantity']] >= $item_quantity) {
                            $isNeed = true;
                            break;
                        }
                    }
                    return $isNeed;
                });
                //需放配件包
                if ($needAccessories->count() > 0) {
                    $discount = 0;
                    $accessoriesData = $group->last();
                    $index = array_search($accessoriesData, $result);
                    if ($index !== false) { //找到原資料，後面插入配件包資料
                        foreach ($acc_config as $acc_cfg) {
                            $item_no = $acc_cfg->ICODE;
                            $item_name = $acc_cfg->INAME;
                            $item_price = $acc_cfg->IUNIT;
                            $discount += $item_price;
                            $accessoriesData[$columns['item_no']] = $item_no;
                            $accessoriesData[$columns['item_name']] = $item_name;
                            $accessoriesData[$columns['item_option_name']] = $item_name;
                            $accessoriesData[$columns['item_price']] = $item_price;
                            $accessoriesData[$columns['item_discount']] = $item_price;
                            $accessoriesData[$columns['item_quantity']] = 1;
                            $result = $this->array_insert($result, $index, $accessoriesData);
                            $index++;
                        }
                        //修改賣家折扣，以抵銷配件包價錢
                        $result = collect($result)->map(function ($item) use ($columns, $orderNo, $discount) {
                            if ($item[$columns['order_no']] == $orderNo) {
                                $item[$columns['discount']] += $discount;
                            }
                            return $item;
                        })->toArray();
                    }
                }
            }
        }

        $this->baseRepository->CreateOrderSampleExportLog($sDate, $eDate, $type, json_encode($result));

        return $result;
    }

    /**
     * 特殊訂單轉檔下載
     *
     * @return array
     */
    public function GetOrderSampleDownload()
    {
        $result = OrderSampleFile::orderBy('created_at', 'DESC')->paginate(10);
        foreach ($result as $item) {
            $start_date = (new Carbon($item->start_date))->format('Y-m-d');
            $end_date = (new Carbon($item->end_date))->format('Y-m-d');
            $fileName = $start_date . '_' . $end_date;
            $item->start_date = $start_date;
            $item->end_date = $end_date;
            $item->color = '';
            switch ($item->convert_type) {
                case '1':
                    $item->convert_type = '廠務可隨時出貨訂單(A)';
                    $fileName .= '廠務可隨時出貨訂單(A)';
                    break;
                case '2':
                    $item->convert_type = '業務同意可以出貨訂單(B)';
                    $fileName .= '業務同意可以出貨訂單(B)';
                    $item->color = 'table-primary';
                    break;
            }
            $item->convert_file_name = $fileName;
        }

        return ['data' => $result];
    }

    /**
     * 取得需額外放感應燈的訂單
     *
     * @return array
     */
    public function GetBulbOrder()
    {
        //erp訂單日期格式
        $date = new DateTime("now");
        //今日
        $eDate = $this->GetTWDateStr($date);
        //今日往前15天
        $date->modify("-15 day");
        $sDate = $this->GetTWDateStr($date);

        $db = $this->baseRepository->GetBulbOrder($sDate, $eDate);
        $result = array();
        foreach ($db->groupBy('OCOD1') as $orderNo => $group) {
            $bulbNum = floor($group->sum('OINUM') / 10); //每滿10個，贈品+1
            if ($bulbNum < 1) { //不須放贈品
                continue;
            } elseif ($group->contains(function ($value, $key) { //訂單已放贈品
                return in_array($value->OICOD, BaseRepository::bulbCode);
            })) {
                continue;
            }
            $result[$orderNo]['bulb'] = $bulbNum;
        }
        //排除銷貨單已有贈品
        if (!empty($result)) {
            $hasBulbOrder = $this->baseRepository->HasBulb(array_keys($result));
            if (!empty($hasBulbOrder)) {
                foreach ($hasBulbOrder as $orderNo) {
                    if (array_key_exists($orderNo, $result)) {
                        unset($result[$orderNo]);
                    }
                }
            }
        }

        return $result;
    }

    /**
     * 取得產品庫存紀錄
     *
     * @param Request $request
     * @return array
     */
    public function GetItemHistory(Request $request)
    {
        $result = array();
        $result['chart'] = ['x' => ['x'], 'y' => [''], 'title' => ''];
        $result['ItemList'] = $this->baseRepository->GetItemAll();
        if (empty($request) || empty($request->id)) {
            return $result;
        }

        $id = $request->id;
        //取得指定日期並轉換格式
        $min = new DateTime('2021-06-29');
        $sDate = $request->start_date;
        if (empty($sDate)) {
            $tmp = new DateTime("now");
            $tmp->modify("-1 month");
        } else {
            $tmp = new DateTime($sDate);
        }
        if ($tmp < $min) {
            $tmp = clone $min;
        }
        $sDate = $this->GetLogDateStr($tmp) . ' 00:00:00';
        $eDate = $request->end_date;
        if (empty($eDate)) {
            $tmp = new DateTime("now");
        } else {
            $tmp = new DateTime($eDate);
        }
        if ($tmp < $min) {
            $tmp = clone $min;
        }
        $eDate = $this->GetLogDateStr($tmp) . ' 23:59:59';

        $xAxis = $this->GetEachDay($sDate, $eDate);
        $yAxis[] = $id;

        $db = $this->baseRepository->GetItemHistory($sDate, $eDate, $id);
        foreach ($xAxis as $value) {
            $target = $db->filter(function ($item) use ($value) {
                return $value == (new DateTime($item->created_at))->format('Y-m-d');
            })->first();
            $y = empty($target) ? 0 : intval($target->INUMB);
            array_push($yAxis, $y);
        }
        array_unshift($xAxis, 'x');
        $result['chart'] = [
            'x' => $xAxis,
            'y' => $yAxis,
            'title' => substr($sDate, 0, 10) . '～' . substr($eDate, 0, 10)
        ];

        return $result;
    }

    /**
     * 取得產品履歷
     *
     * @param Request $request
     * @return array
     */
    public function GetItemLog(Request $request)
    {
        if (empty($request) || empty($request->id)) {
            return array();
        }
        $id = $request->id;
        //取得指定日期並轉換格式
        $sDate = $request->start_date;
        if (empty($sDate)) {
            $tmp = new DateTime("now");
            $tmp->modify("-1 month");
        } else {
            $tmp = new DateTime($sDate);
        }
        $sDate = $this->GetTWDateStr($tmp);
        $eDate = $request->end_date;
        if (empty($eDate)) {
            $tmp = new DateTime("now");
        } else {
            $tmp = new DateTime($eDate);
        }
        $eDate = $this->GetTWDateStr($tmp);

        $result = array();
        $subtotal = 0;
        $db = $this->baseRepository->GetItemLog($sDate, $eDate, $id);
        foreach ($db as $value) {
            $num = $value->Num;
            $type = $value->Type;
            //設定類別CSS、小計計算
            switch ($type) {
                case '進貨':
                    $typeClass = 'Purchase';
                    $subtotal += $num;
                    break;
                case '進貨退回':
                    $typeClass = 'RPurchase';
                    $subtotal -= $num;
                    break;
                case '銷貨':
                    $typeClass = 'Sales';
                    $subtotal -= $num;
                    break;
                case '銷貨退回':
                    $typeClass = 'RSales';
                    $subtotal += $num;
                    break;
                case '調撥':
                    $typeClass = 'Transfer';
                    break;
                case '盤盈':
                    $typeClass = 'InventoryProfit';
                    $subtotal += $num;
                    break;
                case '盤虧':
                    $typeClass = 'InventoryLoss';
                    $subtotal -= $num;
                    break;
                default:
                    $typeClass = '';
                    break;
            }
            //設定數量CSS
            if ($type == '進貨退回' || $type == '銷貨' || $type == '盤虧') {
                $numClass = 'Reduce';
            } else {
                $numClass = '';
            }

            $result[] = [
                'Time' => $this->GetTimeIgnoreMS($value->Time),
                'Id' => (string)$value->Id,
                'Type' => $type,
                'TypeClass' => $typeClass,
                'Stock' => $value->Stock,
                'StockClass' => $value->StockCode,
                'Num' => number_format($num),
                'NumClass' => $numClass,
                'UserName' => $value->UserName
            ];
        }
        return ['datas' => $result, 'subtotal' => $subtotal];
    }

    /**
     * 取得廣告花費資料
     *
     * @param Request $request
     * @return array
     */
    public function GetAdCost(Request $request)
    {
        $result = array();

        $now = new DateTime('now');
        $date = new DateTime($request->date);
        if ($date->format('Y-m') == $now->format('Y-m')) { //目前月份
            $now->modify('-1 day'); //因當天資料不齊，故不取
            $sDate = new DateTime($now->format('Y/m/01'));
            $eDate = new DateTime($now->format('Y/m/d')); //取到當天
        } else {
            $sDate = new DateTime($date->format('Y/m/01'));
            $eDate = new DateTime($date->format('Y/m/t')); //取到當月最後一天
        }
        $twSDate = $this->GetTWDateStr($sDate);
        $twEDate = $this->GetTWDateStr($eDate);

        //設定檔
        $config = $this->configService->GetAdCost($date);
        $salesTarget =  $config->salesTarget ?? 4000000; //營業額目標
        $adThreshold = $config->adThreshold ?? 0.13; //廣告門檻
        $chargeThreshold = $config->chargeThreshold ?? 0.035; //手續費門檻
        $activityThresholdConfig = $config->activityThreshold; //活動費門檻
        $bonusConfig = $config->bonus ?? 0.1; //獎金比例
        $customer = collect($this->configService->GetConfig('customer')); //客戶代碼

        //營業額
        $sales = $this->baseRepository->GetSalesOfCustomer($twSDate, $twEDate);
        $salesTotal = $this->baseRepository->GetSalesTotalTax($twSDate, $twEDate); //已達成的營業額
        $salesDiff = $salesTotal >= $salesTarget ? 0 : ($salesTarget - $salesTotal); //未達成的營業額

        //廣告花費
        $adCost = array();
        $adCostDb = $this->baseRepository->GetAdCost($this->GetLogDateStr($sDate), $this->GetLogDateStr($eDate));
        $shopee_1 = $adCostDb->sum('shopee_1');
        if ($shopee_1 > 0) {
            $adCost[] = ['name' => '蝦皮一店 廣告費', 'cost' => $shopee_1];
        }
        $shopee_2 = $adCostDb->sum('shopee_2');
        if ($shopee_2 > 0) {
            $adCost[] = ['name' => '蝦皮二店 廣告費', 'cost' => $shopee_2];
        }
        $shopee_3 = $adCostDb->sum('shopee_3');
        if ($shopee_3 > 0) {
            $adCost[] = ['name' => '蝦皮三店 廣告費', 'cost' => $shopee_3];
        }
        $ruten = $adCostDb->sum('ruten');
        if ($ruten > 0) {
            $adCost[] = ['name' => '露天 廣告費', 'cost' => $ruten];
        }
        $official = $adCostDb->sum('official');
        if ($official > 0) {
            $adCost[] = ['name' => '官網 廣告費', 'cost' => $official];
        }

        //手續費
        $adCost[] = ['name' => '手續費', 'cost' => $salesTotal * $chargeThreshold];

        //活動費
        if (!empty($activityThresholdConfig)) {
            foreach ($activityThresholdConfig as $value) {
                $code = $value->key;
                $val = $value->value;

                //取得客戶姓名
                $custConfig = $customer->firstWhere('key', $code);
                if (empty($custConfig)) {
                    $name = '未知';
                } else {
                    $name = $custConfig->value;
                }
                $value->text = $name;
                //費用
                $target = $sales->firstWhere('HPCOD', $code);
                if (!empty($target)) {
                    $cost = $target->HTOTA * $val;
                    $adCost[] = ['name' => $name . ' 活動費', 'cost' => $cost];
                }
            }
        }

        //廣告花費統計
        $adCostTotal = collect($adCost)->sum('cost');

        //廣告預算
        $adBudget =  $salesTarget * $adThreshold;
        $adRemainingBudget = $adBudget - $adCostTotal;

        //獎金
        $bonus = 0;
        $bonusLeft = 0;
        if ($salesDiff == 0 && $adRemainingBudget > 0) {
            $bonus = $adRemainingBudget * $bonusConfig;
            $bonusLeft = $adRemainingBudget * (1 - $bonusConfig);
        }

        //ViewData
        $result = [
            'sDate' => $this->GetLogDateStr($sDate),
            'eDate' => $this->GetLogDateStr($eDate),
            'sales' => [
                'total' => $salesTotal,
                'diff' => $salesDiff
            ],
            'adCost' => $adCost,
            'adCostTotal' => $adCostTotal,
            'adBudget' => ['budget' => $adBudget, 'remaining' => $adRemainingBudget],
            'bonus' => ['value' => $bonus, 'left' => $bonusLeft],
            'config' => [
                'salesTarget' => $salesTarget,
                'adThreshold' => $adThreshold,
                'chargeThreshold' => $chargeThreshold,
                'activityThreshold' => $activityThresholdConfig,
                'bonus' => $bonusConfig
            ],
        ];

        return $result;
    }

    /**
     * 取得廣告效益
     *
     * @param array $data
     * @return array
     */
    public function GetAdProfit($data)
    {
        $result = array();
        if (empty($data)) {
            return $result;
        }
        try {
            $result['exportTime'] = $data['exportTime'];
            $result['range'] = $data['range'];
            $adItem = $this->baseRepository->GetAdItem();
            $result['itemDate'] = $adItem->first()->shopee_created_at;
            $itemAllCost = $this->baseRepository->GetItemAllCost();
            $rangeArry = explode('-', $data['range']);
            $sDate = $this->GetTWDateStr(new DateTime(trim($rangeArry[0])));
            $eDate = $this->GetTWDateStr(new DateTime(trim($rangeArry[1])));
            $erpSalesTotalDb = $this->baseRepository->GetSalesOfItem($sDate, $eDate);

            foreach ($data['data'] as  $productId => $value) {
                $target = $adItem->where('shopee', $productId);
                if ($target->count() == 0) {
                    continue;
                } else {
                    $idArray =  $target->pluck('id')->toArray();
                }

                //ERP銷售
                $erp = $erpSalesTotalDb->whereIn('HICOD', $idArray);
                if ($erp->count() > 0) {
                    $erpSalesTotal = $erp->sum('HTOTA');
                    $erpItemCost = $erp->sum('COST');
                    $erpSalesTotalCount = $erp->sum('HINUM');
                } else {
                    $erpSalesTotal = 0;
                    $erpItemCost = 0;
                    $erpSalesTotalCount = 0;
                }

                $cost = $value['cost'];
                $salesAd = $value['salesAd'];
                $salesTotal = $value['salesTotal'];
                $salesTotalCount = $value['salesTotalCount'];
                $salesNature = $salesTotal - $salesAd;
                if ($cost != 0) {
                    $roasAd = $salesAd / $cost;
                    $roasNature = $salesNature / $cost;
                    $roasTotal = $salesTotal / $cost;
                    $erpRoasTotal = $erpSalesTotal / $cost;
                }

                //Roi
                $items = $itemAllCost->whereIn('ICODE', $idArray);
                if ($items->count() > 0) {
                    $itemCost = $items->avg('ISOUR'); //單個產品平均成本
                    if ($salesTotalCount > 0) {
                        $roi = ($salesTotal - ($itemCost * $salesTotalCount)) / ($itemCost * $salesTotalCount) * 100;
                    }
                }

                //ERP Roi
                if ($erpItemCost > 0) {
                    $erpRoi = ($erpSalesTotal - $erpItemCost) / $erpItemCost * 100;
                }

                $result['data'][] = [
                    'productId' => $productId, //商品ID
                    'items' => $items, //ERP料號清單
                    'cost' => $cost, //花費
                    'salesAd' => $salesAd, //廣告效益銷售額
                    'salesNature' => $salesNature, //自然流量銷售額
                    'salesTotal' => $salesTotal, //總銷售額
                    'erpSalesTotal' => $erpSalesTotal, //ERP總銷售額
                    'salesTotalCount' => $salesTotalCount, //銷售數
                    'erpSalesTotalCount' => $erpSalesTotalCount, //ERP銷售數
                    'itemCost' => isset($itemCost) ? ($itemCost * $salesTotalCount) : null, //成本
                    'erpItemCost' => $erpItemCost, //ERP成本
                    'roasAd' => $roasAd,
                    'roasNature' => $roasNature,
                    'roasTotal' => $roasTotal,
                    'roi' => $roi,
                    'erpRoasTotal' => $erpRoasTotal,
                    'erpRoi' => $erpRoi,
                ];
            }
        } catch (Exception $ex) {
            return $result;
        }

        return $result;
    }

    /**
     * 取得廣告花費匯出資料
     *
     * @param Request $request
     * @return array
     */
    public function GetAdCostExport(Request $request)
    {
        $result = [['日期', '蝦皮一店', '蝦皮二店', '蝦皮三店', '露天', '官網']];

        $sDate = $request->export_start_date;
        $eDate = $request->export_end_date;

        $adCostDb = $this->baseRepository->GetAdCost($sDate, $eDate);
        $dayList = $this->GetEachDayByMonth(new DateTime($sDate));
        foreach ($dayList as $day) {
            $adCost = $adCostDb->firstWhere('src_date', $day);
            if (!empty($adCost)) {
                $result[] = [
                    $day,
                    $adCost->shopee_1,
                    $adCost->shopee_2,
                    $adCost->shopee_3,
                    $adCost->ruten,
                    $adCost->official
                ];
            } else {
                $result[] = [$day, '0', '0', '0', '0', '0'];
            }
        }
        return $result;
    }

    /**
     * 取得日報表資料
     *
     * @param Request $request
     * @return array
     */
    public function GetProductionDiary(Request $request)
    {
        $now = Carbon::today()->format('Y-m-d');
        $sDate = $request->get('start_date', Carbon::today()->format('Y-m-d'));
        $eDate = $request->get('end_date', Carbon::today()->format('Y-m-d'));

        try {
            //取得驗貨紀錄，並以日期分組
            $checkData = collect();
            $PoseinCheckDataLog = PoseinCheckDataLog::whereBetween('CREATE_Date', [$sDate . ' 00:00:00', $eDate . ' 23:59:59'])->where('CheckCount', 1)->get();
            $period = CarbonPeriod::create($sDate, $eDate)->toArray();
            foreach ($period as $date) {
                $dateStart = $date->format('Y-m-d 00:00:00');
                $dateLastTime = $date->format('Y-m-d 23:59:59');
                $checkData[$date->format('Y-m-d')] = $PoseinCheckDataLog->whereBetween('CREATE_Date', [$dateStart, $dateLastTime]);
            }

            $diaryData = ProductionDiary::all();
            $finishedDates = $diaryData->pluck('settle_date')->toArray();
            $diaryData = $diaryData->whereBetween('settle_date', [$sDate, $eDate])->sortByDesc('settle_date');
            foreach ($diaryData as $val) {
                if (isset($checkData[$val->settle_date])) {
                    $checkCount = $checkData[$val->settle_date]->count();
                    $lastCheckTime = substr($checkData[$val->settle_date]->max('CREATE_Date'), 0, 19);
                } else {
                    $checkCount = '';
                    $lastCheckTime = '';
                }
                $val->checkCount = $checkCount;
                $val->lastCheckTime = $lastCheckTime;
                $val->canModify = $val->settle_date >= $now; //大於等於今天才可以做修改
                //狀態判斷
                $status = 0;
                if (empty($checkCount) || empty($lastCheckTime)) { //無驗貨紀錄
                    $status = 1;
                } else if ($checkCount > $val->amount && $lastCheckTime < $val->estimated_time) { //實際驗貨數量>預期 並且 實際驗貨時間<預期，符合預期
                    $status = 2;
                } else if (!empty($val->signinger)) { //不符合預期，已覆核
                    $status = 3;
                } else { //不符合預期，未覆核
                    $status = 4;
                }
                $val->status = $status;
            }
        } catch (Exception $ex) {
            $diaryData = collect();
            $finishedDates = [];
        }

        return [
            'data' => $diaryData,
            'finishedDates' => $finishedDates,
            'users' => $this->configService->GetConfig('users'),
        ];
    }

    /**
     * 取得當日產品包裝進度
     *
     * @return array
     */
    public function GetProductionCurrentProgress()
    {
        $today = Carbon::today()->format('Y-m-d');
        try {
            $target = ProductionDiary::where('settle_date', $today)->first()->amount ?? 0;
        } catch (Exception $ex) {
            $target = 0;
        }
        $finished = PoseinCheckDataLog::whereBetween('CREATE_Date', [$today . ' 00:00:00', $today . ' 23:59:59'])->where('CheckCount', 1)->count();

        return [
            'target' => $target,
            'finished' => $finished,
        ];
    }

    /**
     * 取得客訴單
     *
     * @param Request $request
     * @return array
     */
    public function GetCustomerComplaint(Request $request)
    {
        $sDate = $request->get('start_date', Carbon::today()->addDays(-15)->format('Y-m-d')) . ' 00:00:00';
        $eDate = $request->get('end_date', Carbon::today()->format('Y-m-d')) . ' 23:59:59';
        $data = collect();
        try {
            $data = CustomerComplaint::whereBetween("updated_at", [$sDate, $eDate])->orderby('updated_at', 'DESC');
            $orderNo = $request->get('order_no');
            if (!empty($orderNo)) { //搜尋指定，含作廢
                $data->where(function ($query) use ($orderNo) {
                    $query
                        ->where('order_no', $orderNo)
                        ->orWhere('origin_no', $orderNo);
                });
            } else { //搜尋全部，不含作廢
                $data->where('status', '!=', '99');
                //是否含結案
                $includeDone = $request->get('include_done', 0);
                if (empty($includeDone)) {
                    $data->where('status', '!=', '0');
                }
            }
            $data = $data->get();
        } catch (Exception $ex) {
        }

        //找到訂單客戶代碼
        $orderData =  Order::whereIn('OCOD1', $data->unique('order_no')->pluck('order_no')->toArray())->select('OCOD1', 'OCCOD')->get();
        foreach ($orderData as $order) {
            $tmp = $data->firstWhere('order_no', $order->OCOD1);
            if (!empty($tmp)) {
                $tmp->OCCOD = $order->OCCOD;
            }
        }

        //對應狀態代碼
        $config = $this->configService->GetConfig('customer_complaint_status');
        foreach ($data as $d) {
            if (count($config) > 0) {
                $d->status_name = collect($config)->firstWhere('key', $d->status)->value ?? 'Undefined';
            } else {
                $d->status_name = 'Undefined';
            }
            $d->elapsed_days = Carbon::today()->diff($d->updated_at)->days;
            $d->is_timeout = $d->status != 0 && $d->status != 99 && $d->elapsed_days > 3;
        }

        return ['data' => $data, 'config' => $config];
    }

    /**
     * 是否有包驗貨排班
     *
     * @return boolean
     */
    public function HasPackageShift()
    {
        return $this->baseRepository->HasPackageShift(Carbon::today()->format('Y-m-d'));
    }

    /**
     * 取得前15天到今天的訂單資料
     *
     * @return array
     */
    private function GetOrder()
    {
        //erp訂單日期格式
        $date = new DateTime("now");
        //今日
        $eDate = $this->GetTWDateStr($date);
        //今日往前15天
        $date->modify("-15 day");
        $sDate = $this->GetTWDateStr($date);

        $db = $this->baseRepository->GetOrder($sDate, $eDate);

        //取得各料號、品名，的訂單需求量、訂單號
        $result = array();
        foreach ($db as $value) {
            $itemNo = (string)$value->OICOD; //料號
            $itemName = $value->OINAM; //品名
            $orderNo = (string)$value->OCOD1; //訂單號
            $orderNeedNum = ceil($value->OINUM); //訂單需求數量
            $orignalOrderNo = (string)$value->OCOD4; //原始訂單號

            $item = &$result[$itemNo][$itemName];
            if (empty($item)) {
                $item['amount'] = $orderNeedNum;
                $item['order'][$orderNo] = $orignalOrderNo;
            } else {
                $item['amount'] += $orderNeedNum;
                if (!array_key_exists($orderNo, $item['order'])) {
                    $item['order'][$orderNo] = $orignalOrderNo;
                }
            }
        }

        return $result;
    }

    /**
     * 取得料號在各庫存的數量
     *
     * @param array $icodeList 料號清單
     * @return array
     */
    private function GetStock(array $icodeList = null)
    {
        $db = $this->baseRepository->GetStock($icodeList);
        $result = array();
        foreach ($db as $value) {
            $itemNo = (string)$value->SITEM; //料號
            $itemNum = $value->SNUMB; //庫存量
            $wareHouseName = $value->STNAM; //倉庫名稱

            //加總
            $total = &$result[$itemNo]['total'];
            if (empty($total)) {
                $total = $itemNum;
            } else {
                $total += $itemNum;
            }

            $result[$itemNo][$wareHouseName] = $itemNum;
        }

        return $result;
    }

    /**
     * 取得指定料號，未進貨採購單號
     *
     * @param array $icodeList 產品料號清單
     * @return array
     */
    private function GetPurchaseNotIn(array $icodeList)
    {
        $result = array();
        if (empty($icodeList)) {
            return $result;
        }
        $db = $this->baseRepository->GetPurchaseNotIn($icodeList);
        foreach ($db->groupBy('SICOD') as $itemNo => $group) {
            foreach ($group as $value) {
                $result[(string)$itemNo][] = (string) $value->SCOD1;
            }
        }
        return $result;
    }

    /**
     * 取得指定月份的每天日期
     *
     * @param DateTime $date
     * @return array
     */
    private function GetEachDayByMonth(DateTime $date)
    {
        $result = array();

        $startTime = new DateTime($date->format('Y-m-01'));
        $endTime = new DateTime($date->format('Y-m-t'));
        while ($startTime <= $endTime) {
            $result[] = $startTime->format('Y-m-d');
            $startTime->modify('1 day');
        }

        return $result;
    }

    /**
     * 取得指定區間的每天日期
     *
     * @param string $sDate 起日。yyyy/mm/dd hh:mm:ss
     * @param string $eDate 訖日。yyyy/mm/dd hh:mm:ss
     * @return array
     */
    private function GetEachDay(string $sDate, string $eDate)
    {
        $result = array();
        if (empty($sDate) || empty($eDate)) {
            return $result;
        }
        $start = new DateTime($sDate);
        $end = new DateTime($eDate);
        if ($start > $end) { //起訖相反，互換日期
            $tmp = $start;
            $start = $end;
            $end = $tmp;
        }

        while ($start <= $end) {
            $result[] = $start->format('Y-m-d');
            $start->modify('+1 day');
        }

        return $result;
    }

    /**
     * 取得指定區間的每小時時間
     *
     * @param string $sDate 起日。yyyy/mm/dd hh:mm:ss
     * @param string $eDate 訖日。yyyy/mm/dd hh:mm:ss
     * @return array
     */
    private function GetEachHour(string $sDate, string $eDate)
    {
        $result = array();
        if (empty($sDate) || empty($eDate)) {
            return $result;
        }
        $start = new DateTime($sDate);
        $end = new DateTime($eDate);
        if ($start > $end) { //起訖相反，互換日期
            $tmp = $start;
            $start = $end;
            $end = $tmp;
        }

        while ($start <= $end) {
            $result[] = $start->format('H:00');
            $start->modify('+1 hour');
        }

        return $result;
    }

    /**
     * 取得託運狀態
     *
     * @param string $remark 備註內容
     * @return string
     */
    private function GetFreightStatus(string $remark)
    {
        if (empty($remark)) {
            return '';
        }
        $start = mb_strpos($remark, '託運#', 0, 'utf-8');
        if ($start === false) { //未含'託運#'字樣
            return '';
        }
        $status = trim(mb_substr($remark, $start + 3, 1, 'utf-8'));
        if (!empty($status) && strlen($status) == 1 && is_numeric($status)) { //非空白、長度1、數值
            return $status;
        } else {
            return '';
        }
    }

    /**
     * 判斷請求是否為重物
     *
     * @param \Illuminate\Http\Request $request
     * @return bool
     */
    private function IsHeavy(Request $request)
    {
        //判斷是否為重物
        if (empty($request) || empty($request->type)) {
            $isHeavy = false;
        } else {
            $isHeavy = $request->type == '1';
        }
        return $isHeavy;
    }

    /**
     * 判斷字串陣列是否有統編敘述
     *
     * @param array $strList
     * @return bool
     */
    private function IsContainsTaxID(array $strList)
    {
        if (empty($strList)) {
            return false;
        } else {
            foreach ($strList as $value) {
                if (strstr($value, '統編') || strstr($value, '統一編號')) {
                    return true;
                }
            }
            return false;
        }
    }

    /**
     * array insert
     *
     * @param array $array
     * @param integer $index
     * @param [type] $insertData
     * @return array
     */
    private function array_insert(array $array, int $index, $insertData)
    {
        if (empty($array) || $index === false) {
            return $array;
        }
        return array_merge(array_slice($array, 0, $index + 1), [$insertData], array_slice($array, $index + 1));
    }
}
